@@ -4,26 +4,18 @@
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
 #
-# This toolkit is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by the Free
-# Software Foundation; either version 2 of the License, or (at your option)
+# This file is free software; you can redistribute it and/or modify it
+# under the terms of the GNU Lesser General Public License as published by the
+# Free Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
 #
-# PyMca is distributed in the hope that it will be useful, but WITHOUT ANY
+# This file is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 # details.
 #
-# You should have received a copy of the GNU General Public License along with
-# PyMca; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-#
-# PyMca follows the dual licensing model of Riverbank's PyQt and cannot be
-# used as a free plugin for a non-free program.
-#
-# Please contact the ESRF industrial unit (industry@esrf.fr) if this license
-# is a problem for you.
 #############################################################################*/
+__author__ = "V.A. Sole - ESRF Data Analysis"
 import os
 import numpy
 import time
@@ -397,8 +389,8 @@ def save3DArrayAsHDF5(data, filename, axes=None, labels=None, dtype=None, mode='
             #should I raise an error?
             pass
 
-        nxEntry['title'] = "PyMca saved 3D Array".encode('utf-8')
-        nxEntry['start_time'] = getDate().encode('utf-8')
+        nxEntry['title'] = numpy.string_("PyMca saved 3D Array".encode('utf-8'))
+        nxEntry['start_time'] = numpy.string_(getDate().encode('utf-8'))
         nxData = nxEntry.require_group('NXdata')
         if ('NX_class' not in nxData.attrs):
             nxData.attrs['NX_class'] = 'NXdata'.encode('utf-8')
@@ -551,7 +543,7 @@ def save3DArrayAsHDF5(data, filename, axes=None, labels=None, dtype=None, mode='
             adset[:] = dim[:]
             adset.attrs['axis'] = i + 1
         dset.attrs['axes'] = (":".join(axesAttribute)).encode('utf-8')
-        nxEntry['end_time'] = getDate().encode('utf-8')
+        nxEntry['end_time'] = numpy.string_(getDate().encode('utf-8'))
         if mode.lower() == 'nexus+':
             #create link
             g = h5py.h5g.open(hdf.fid, '/'.encode('utf-8'))
