@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2013 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -24,12 +24,13 @@
 # Please contact the ESRF industrial unit (industry@esrf.fr) if this license
 # is a problem for you.
 #############################################################################*/
-__revision__ = "$Revision: 1.26 $"
+__revision__ = "$Revision: 1.27 $"
 __author__ = "V.A. Sole - ESRF Software Group"
-from PyMca import Elements
+import sys
 import copy
 import numpy
-
+from PyMca import Elements
+from PyMca.XRFMC import XRFMCHelper
 
 class ConcentrationsConversion(object):
     def getConcentrationsAsHtml(self, concentrations=None):
@@ -140,9 +141,9 @@ class ConcentrationsConversion(object):
         for mmolarflag in mmolarflaglist:
             labels = ['Element', 'Group', 'Fit_Area', 'Sigma_Area']
             if mmolarflag:
-                labels += 'mM_Concentration'
+                labels += ['mM_Concentration']
             else:
-                labels += 'Mass_fraction'
+                labels += ['Mass_fraction']
             if 'layerlist' in result:
                 if type(result['layerlist']) != type([]):
                     result['layerlist'] = [result['layerlist']]
@@ -189,6 +190,7 @@ class ConcentrationsTool(object):
         self.config['usematrix'] = 0
         self.config['useattenuators'] = 1
         self.config['usemultilayersecondary'] = 0
+        self.config['usexrfmc'] = 0
         self.config['flux'] = 1.0E10
         self.config['time'] = 1.0
         self.config['area'] = 30.0
@@ -208,7 +210,8 @@ class ConcentrationsTool(object):
         return copy.deepcopy(self.config)
 
     def processFitResult(self, config=None, fitresult=None,
-                         elementsfrommatrix=False, fluorates=None):
+                         elementsfrommatrix=False, fluorates=None,
+                         addinfo=False):
         # I should check if fit was successful ...
         if fitresult is None:
             fitresult = self.fitresult
@@ -220,7 +223,13 @@ class ConcentrationsTool(object):
             self.config = config
         if 'usemultilayersecondary' not in self.config:
             self.config['usemultilayersecondary'] = 0
+        if 'usexrfmc' not in self.config:
+            self.config['usexrfmc'] = 0
         secondary = self.config['usemultilayersecondary']
+        xrfmcSecondary = self.config['usexrfmc']
+        if secondary and xrfmcSecondary:
+            txt = "Only one of built-in secondary and Monte Carlo correction can be used"
+            raise ValueError(txt) 
         # get attenuators and matrix from fit
         attenuators = []
         beamfilters = []
@@ -269,6 +278,9 @@ class ConcentrationsTool(object):
             multilayer = [matrix]
             if not Elements.isValidMaterial(matrix[0]):
                 raise ValueError("Material %s is not defined" % matrix[0])
+        if xrfmcSecondary and (len(layerlist) > 1):
+            txt = "Multilayer Monte Carlo correction not implemented yet"
+            raise ValueError(txt)
         energyList = fitresult['result']['config']['fit']['energy']
         if energyList is None:
             raise ValueError("Invalid energy")
@@ -459,27 +471,46 @@ class ConcentrationsTool(object):
                         fom = newfom
                         referenceElement = key
                 referenceLayers  = referenceLayerDict[referenceElement]
-            solidangle    = 1.0
+
+            referenceTransitions = None
             for group in groupsList:
                 item = group.split()
                 element = item[0]
-                transitions = item[1] + " xrays"
                 if element == referenceElement:
+                    transitions = item[1] + " xrays"
+                    if referenceTransitions is None:
+                        referenceTransitions = transitions
+                        referenceGroup = group
+                    elif (referenceTransitions[0] == transitions[0]) and\
+                         (referenceTransitions[0] == 'L'):
+                        # this prevents selecting L1 and selects L3 although
+                        # given the appropriate area, L2 can be a safer choice.
+                        referenceGroup = group
+                        referenceTransitions = transitions
+                elif referenceTransitions is not None:
                     break
             theoretical = 0.0
             for ilayer in referenceLayers:
                 if elementsfrommatrix:
-                    theoretical += fluolist[ilayer][referenceElement]['rates'][transitions] * \
+                    theoretical += fluolist[ilayer][referenceElement]['rates'][referenceTransitions] * \
                                    fluolist[ilayer][referenceElement]['mass fraction']
                 else:
                     theoretical  += materialComposition[ilayer][referenceElement] * \
-                                    fluolist[ilayer][referenceElement]['rates'][transitions]
+                                    fluolist[ilayer][referenceElement]['rates'][referenceTransitions]
             if theoretical <= 0.0:
                 raise ValueError(\
                     "Theoretical rate is almost 0.0 Impossible to determine flux")
             else:
-                flux = fitresult['result'][group]['fitarea'] / theoretical
+                if (config['distance'] > 0.0) and (config['area'] > 0.0):
+                    #solidangle = config['area']/(4.0 * numpy.pi * pow(config['distance'],2))
+                    radius2 = config['area']/numpy.pi
+                    solidangle = 0.5 * (1.0 -  (config['distance']/numpy.sqrt(pow(config['distance'],2)+ radius2)))
+                else:
+                    solidangle = 1.0
+                flux = fitresult['result'][referenceGroup]['fitarea'] / (theoretical * solidangle)
         else:
+            referenceElement = None
+            referenceTransitions = None
             #solidangle = config['area']/(4.0 * numpy.pi * pow(config['distance'],2))
             radius2 = config['area']/numpy.pi
             solidangle = 0.5 * (1.0 -  (config['distance']/numpy.sqrt(pow(config['distance'],2)+ radius2)))
@@ -609,8 +640,61 @@ class ConcentrationsTool(object):
                     for layer in ddict['layerlist']:
                         if group in ddict[layer]['area'].keys():
                             ddict['area'][group] += ddict[layer]['area'][group]
+        if xrfmcSecondary and (not elementsfrommatrix):
+            xrfmcCorrections = None
+            if 'xrfmc' in fitresult:
+                xrfmcCorrections = fitresult['xrfmc'].get('corrections', None)
+            if xrfmcCorrections is None:
+                if 'xrfmc' in fitresult['result']:
+                    xrfmcCorrections = fitresult['result']['xrfmc'].get('corrections', None)
+            if xrfmcCorrections is None:
+                # try to see if they were in the configuration
+                if 'xrfmc' in fitresult['result']['config']:
+                    xrfmcCorrections = fitresult['result']['config']['xrfmc'].get('corrections',
+                                                                                  None)
+            if xrfmcCorrections is None:
+                # calculate the corrections
+                xrfmcCorrections = XRFMCHelper.getXRFMCCorrectionFactors(fitresult['result']['config'])
 
-        return ddict
+            
+            if referenceElement is not None:
+                referenceLines = referenceTransitions.split()[0]
+                referenceCorrection = xrfmcCorrections[referenceElement][referenceLines]\
+                                            ['correction_factor'][-1]
+                xrfmcCorrections[referenceElement][referenceLines]\
+                                            ['correction_factor'][-1] = 1.0
+                for group in groupsList:
+                    item = group.split()
+                    element = item[0]
+                    lines = item[1]
+                    if element in xrfmcCorrections:
+                        if element != referenceElement:
+                            if lines != referenceLines:
+                                xrfmcCorrections[element][lines]['correction_factor'][-1] *= referenceCorrection
+            # now we have to apply the corrections
+            for group in groupsList:
+                item = group.split()
+                element = item[0]
+                lines = item[1]
+                if element in xrfmcCorrections:
+                    correction = xrfmcCorrections[element][item[1]]['correction_factor'][-1]
+                    ddict['mass fraction'][group] /= correction
+        if addinfo:
+            addInfo = {}
+            addInfo['ReferenceElement'] = referenceElement
+            addInfo['ReferenceTransitions'] = referenceTransitions
+            addInfo['SolidAngle'] = solidangle
+            if config['time'] > 0.0:
+                addInfo['Time'] = config['time']
+            else:
+                addInfo['Time'] = 1.0
+            addInfo['Flux'] = flux / addInfo['Time']       
+            addInfo['I0'] = flux
+            addInfo['DetectorDistance'] = config['distance']
+            addInfo['DetectorArea'] = config['area']
+            return ddict , addInfo
+        else:
+            return ddict
 
     def _figureOfMerit(self, element, fluo, fitresult):
         weight = 0.0
@@ -631,7 +715,6 @@ class ConcentrationsTool(object):
                         weight = weightHelp
         return weight
 
-
 def main():
     import sys
     import getopt
