@@ -116,10 +116,10 @@ class ExternalImagesWindow(MaskImageWidget.MaskImageWidget):
                                      self._flipLeftRight)
             self._flipMenu.addAction(QString("Flip Image Up-Down"),
                                      self._flipUpDown)
-        else:
-            self.connect(self.graphWidget.hFlipToolButton,
-                 qt.SIGNAL("clicked()"),
-                 self.__hFlipIconSignal)
+        #else:
+        #    self.connect(self.graphWidget.hFlipToolButton,
+        #         qt.SIGNAL("clicked()"),
+        #         self.__hFlipIconSignal)
 
 
     def sizeHint(self):
@@ -406,6 +406,9 @@ class ExternalImagesWindow(MaskImageWidget.MaskImageWidget):
                                                   replot=replot,
                                                   replace=replace)
 
+    def getCurrentIndex(self):
+        return self.slider.value()
+
 
 def test():
     app = qt.QApplication([])
