@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2013 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -40,7 +40,14 @@ from PyMca import AttenuatorsTable
 from PyMca import ConcentrationsWidget
 from PyMca import EnergyTable
 from PyMca import PyMcaDirs
+XRFMC_FLAG = False
 if QTVERSION > '4.0.0':
+    try:
+        import PyMca.XRFMC.XRFMCPyMca as XRFMCPyMca
+        XRFMC_FLAG = True
+    except ImportError:
+        # no XRFMC support
+        pass
     from PyMca import StripBackgroundWidget
     # This strange looking import is to workaround an endless import
     SCANWINDOW = False
@@ -204,10 +211,20 @@ class FitParamWidget(FitParamForm):
                 tabCompoundFitLayout.setSpacing(6)
                 self.compoundFitWidget   = AttenuatorsTable.CompoundFittingTab(self.tabCompoundFit,
                                                                                "tabCompound_fit")
-                tabConcentrationsLayout.addWidget(self.compoundFitWidget,0,0)
-                self.mainTab.addTab(self.tabConcentrations, str("COMPOUND FIT"))
+                tabCompoundFitLayout.addWidget(self.compoundFitWidget,0,0)
+                self.mainTab.addTab(self.tabCompoundFit, str("COMPOUND FIT"))
             #end compound fit tab
 
+        if XRFMC_FLAG:
+            self.tabXRFMC =  qt.QWidget()
+            tabXRFMCLayout = qt.QGridLayout(self.tabXRFMC)
+            tabXRFMCLayout.setMargin(11)
+            tabXRFMCLayout.setSpacing(6)
+            self.tabXRFMCWidget   = XRFMCPyMca.XRFMCTabWidget(\
+                                                self.tabXRFMC)
+            tabXRFMCLayout.addWidget(self.tabXRFMCWidget,0,0)
+            self.mainTab.addTab(self.tabXRFMC, str("XRFMC"))
+
         self.layout().setMargin(0)
 
         #I had to add this line to prevent a crash. Why?
@@ -257,11 +274,15 @@ class FitParamWidget(FitParamForm):
         self.input = None
         self.linpolOrder= None
         self.exppolOrder= None
-        self.setParameters(pardict={'attenuators':{'Air'       :[0,"Air",0.001204790,1.0],
-                                                   'Contact'   :[0,"Au1",19.370,1.0E-06],
-                                                   'Deadlayer' :[0,"Si1",2.330,0.0020],
-                                                   'Window'    :[0,"Be1",1.848,0.0100]},
-                                    'concentrations':self.concentrationsWidget.getParameters()})
+        pardict={'attenuators':{'Air'       :[0,"Air",0.001204790,1.0],
+                                'Contact'   :[0,"Au1",19.370,1.0E-06],
+                                'Deadlayer' :[0,"Si1",2.330,0.0020],
+                                'Window'    :[0,"Be1",1.848,0.0100]},
+                 'concentrations':self.concentrationsWidget.getParameters()}             
+        if XRFMC_FLAG:
+            pardict = self.tabXRFMCWidget.getParameters()
+        self.setParameters(pardict=pardict)
+        
         
 
         self.prevTabIdx= None
@@ -421,6 +442,9 @@ class FitParamWidget(FitParamForm):
             self._stripDialog = StripBackgroundWidget.StripBackgroundDialog()
             self._stripDialog.setWindowIcon(qt.QIcon(\
                                 qt.QPixmap(Icons.IconDict["gioconda16"])))
+            if self.height() < 801:
+                self._stripDialog.setMinimumHeight(int(0.85*self.height()))
+                self._stripDialog.setMaximumHeight(int(0.85*self.height()))
 
         self._stripDialog.setParameters(pars)
         self._stripDialog.setData(x, y)
@@ -512,6 +536,9 @@ class FitParamWidget(FitParamForm):
         self.__setPeakShapePar()
         if "tube" in pardict:
             self.xRayTube.setParameters(pardict["tube"])
+        if "xrfmc" in pardict:
+            if XRFMC_FLAG:
+                self.tabXRFMCWidget.setParameters(pardict)
 
     def getParameters(self):
         pars= {}
@@ -519,6 +546,8 @@ class FitParamWidget(FitParamForm):
         sections.append('multilayer')
         sections.append('materials')
         sections.append('tube')
+        if XRFMC_FLAG:
+            sections.append('xrfmc')
         for key in sections:
             pars[key]= self.__getPar(key)
         return pars
@@ -542,6 +571,9 @@ class FitParamWidget(FitParamForm):
             return self.__getTubePar()
         if parname in ["concentrations", "CONCENTRATIONS"]:
             return self.__getConPar()
+        if parname in ["xrfmc", "XRFMC"]:
+            if XRFMC_FLAG:
+                return self.tabXRFMCWidget.getParameters()["xrfmc"]
         return None
 
     def __setAttPar(self, pardict):
