@@ -1,34 +1,27 @@
 #/*##########################################################################
-# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2013 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
 #
-# This toolkit is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by the Free
+# This file is free software; you can redistribute it and/or modify it
+# under the terms of the GNU Lesser General Public License as published by the Free
 # Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
 #
-# PyMca is distributed in the hope that it will be useful, but WITHOUT ANY
+# This file is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 # details.
 #
-# You should have received a copy of the GNU General Public License along with
-# PyMca; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-#
-# PyMca follows the dual licensing model of Riverbank's PyQt and cannot be
-# used as a free plugin for a non-free program.
-#
 # Please contact the ESRF industrial unit (industry@esrf.fr) if this license
 # is a problem for you.
 #############################################################################*/
+__author__ = "V.A. Sole - ESRF Software Group"
+__license__ = "LGPL"
 import numpy
 from numpy.linalg import inv
 import time
-__author__ = "V.A. Sole - ESRF Software Group"
-__revision__ = "$Revision: 1.19 $"
 # codes understood by the routine
 CFREE       = 0
 CPOSITIVE   = 1
@@ -44,9 +37,66 @@ ONED = 0
 def LeastSquaresFit(model, parameters0, data=None, maxiter = 100,constrains=None,
                         weightflag = 0,model_deriv=None,deltachi=None,fulloutput=0,
                         xdata=None,ydata=None,sigmadata=None,linear=None):
+    """
+    Typical use:
+
+    LeastSquaresFit(model_function, parameters, xdata=xvalues, ydata=yvalues)
+
+        model_function - it has the form model_function(parameters, x) where parameters is a sequence
+                         containing the parameters to be refined and x is the array of values in which
+                         the function is to be evaluated.
+
+        parameters - sequence with the initial values to be refined
+
+        xdata - array with the x axis data points
+
+        ydata - array with the y axis data points
+
+    Additional keywords:
+
+        sigmadata - array with the uncertainties associated to ydata (default is sqrt(y) )
+
+        weightflag - 0 Means no weighted fit 1 means weighted fit
+
+        constrains - if provided, it is a 2D sequence of dimension (3, n_parameters) where, for each
+                     parameter denoted by the index i, the meaning is
+
+                     constrains[0][i] -> 0 - Free (Gefit.CFREE)
+                                         1 - Positive (Gefit.CPOSITIVE)
+                                         2 - Quoted (Gefit.CQUOTED)
+                                         3 - Fixed (Gefit.CFIXED)
+                                         4 - Factor (Gefit.CFACTOR)
+                                         5 - Delta (Gefit.CDELTA)
+                                         6 - Sum (Gefit.CSUM)
+
+
+                     constrains[1][i] -> Ignored if constrains[0][i] is 0, 1, 3
+                                         Min value of the parameter if constrains[0][i] is Gefit.CQUOTED
+                                         Index of fitted parameter to which it is related
+
+                     constrains[2][i] -> Ignored if constrains[0][i] is 0, 1, 3
+                                         Max value of the parameter if constrains[0][i] is Gefit.CQUOTED
+                                         Factor to apply to related parameter with index constrains[1][i]
+                                         Difference with parameter with index constrains[1][i]
+                                         Sum obtained when adding parameter with index constrains[1][i]
+
+        model_deriv - function providing the derivatives of the fitting function respect to the fitted parameters.
+                      It will be called as model_deriv(parameters, index, x) where parameters are the current values
+                      of the fitting parameters, index is the fitting parameter index of which the the derivative has
+                      to be provided in the supplied array of x points.
+
+        linear - Flag to indicate a linear fit instead of a non-linear. Default is non-linear fit (=false)
+
+        maxiter - Maximum number of iterations (default is 100)
+
+    Output:
+
+        fitted_parameters, reduced_chi_square, uncertainties
+
+    """
     if constrains is None:
         constrains = []
-    parameters = numpy.array(parameters0).astype(numpy.float)
+    parameters = numpy.array(parameters0, dtype=numpy.float, copy=False)
     if linear is None:linear=0
     if deltachi is None:
         deltachi = 0.01
@@ -61,7 +111,7 @@ def LeastSquaresFit(model, parameters0, data=None, maxiter = 100,constrains=None
         else:
             x=xdata
     if linear:
-           return LinearLeastSquaresFit(model,parameters0,
+           return LinearLeastSquaresFit(model,parameters,
                                         data,maxiter,
                                         constrains,weightflag,model_deriv=model_deriv,
                                         deltachi=deltachi,
@@ -77,7 +127,7 @@ def LeastSquaresFit(model, parameters0, data=None, maxiter = 100,constrains=None
                 constrains[0].append(0)
                 constrains[1].append(0)
                 constrains[2].append(0)
-            return RestreinedLeastSquaresFit(model,parameters0,
+            return RestreinedLeastSquaresFit(model,parameters,
                                     data,maxiter,
                                     constrains,weightflag,
                                     model_deriv=model_deriv,
@@ -90,7 +140,7 @@ def LeastSquaresFit(model, parameters0, data=None, maxiter = 100,constrains=None
             print("You should reconsider how to write your function")
             raise TypeError("You should reconsider how to write your function")
     else:
-        return RestreinedLeastSquaresFit(model,parameters0,
+        return RestreinedLeastSquaresFit(model,parameters,
                                 data,maxiter,
                                 constrains,weightflag,model_deriv=model_deriv,
                                 deltachi=deltachi,
@@ -151,7 +201,7 @@ def LinearLeastSquaresFit(model0,parameters0,data0,maxiter,
             raise ValueError("Linear fit cannot handle quoted constraint")
     # make a local copy of the function for an easy speed up ...
     model = model0
-    parameters = numpy.array(parameters0)
+    parameters = numpy.array(parameters0, dtype=numpy.float, copy=False)
     if data0 is not None:
         selfx = numpy.array([x[0] for x in data0])
         selfy = numpy.array([x[1] for x in data0])
@@ -265,7 +315,7 @@ def RestreinedLeastSquaresFit(model0,parameters0,data0,maxiter,
                 raise ValueError("Unknown constraint %s" % constrains[0][i])
     # make a local copy of the function for an easy speed up ...
     model = model0
-    parameters = numpy.array(parameters0)
+    parameters = numpy.array(parameters0, dtype=numpy.float, copy=False)
     if ONED:
         data = numpy.array(data0)
         x = data[1:2,0]
@@ -462,6 +512,8 @@ def ChisqAlphaBeta(model0, parameters, x,y,weight, constrains,model_deriv=None,l
         pwork [free_index[i]] = fitparam [i]
     newpar = getparameters(pwork.tolist(),constrains)
     newpar = numpy.take(newpar,noigno)
+    if n_free == 0:
+        raise ValueError("No free parameters to fit")
     for i in range(n_free):
         if model_deriv is None:
             #pwork = parameters.__copy__()
