@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2013 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -29,16 +29,18 @@ import numpy
 import h5py
 try:
     from PyMca import DataObject
+    from PyMca import PhysicalMemory
 except ImportError:
     print("HDF5Stack1D importing DataObject from local directory!")
     import DataObject
+    import PhysicalMemory
 try:
     from PyMca import NexusDataSource
 except ImportError:
     print("HDF5Stack1D importing NexusDataSource from local directory!")
     import NexusDataSource
 
-DEBUG = 0    
+DEBUG = 0
 SOURCE_TYPE = "HDF5Stack1D"
 
 class HDF5Stack1D(DataObject.DataObject):
@@ -72,6 +74,10 @@ class HDF5Stack1D(DataObject.DataObject):
                  That means scanlist = ["/whatever1"]
                  and               selection['y'] = "/whatever2/counts"
         """
+        if DEBUG:
+            print("filelist = ", filelist)
+            print("selection = ", selection)
+            print("scanlist = ", scanlist)
         # all the files in the same source
         hdfStack = NexusDataSource.NexusDataSource(filelist)
 
@@ -84,7 +90,7 @@ class HDF5Stack1D(DataObject.DataObject):
                 entryNames.append(key)
 
         # built the selection in terms of HDF terms
-        # for the time being, the only the firs item in x selection used
+        # for the time being, only the first item in x selection used
         xSelection = selection['x']
         if type(xSelection) == type([]):
             if len(xSelection):
@@ -108,6 +114,7 @@ class HDF5Stack1D(DataObject.DataObject):
         else:
             mSelection = None
 
+        USE_JUST_KEYS = False
         # deal with the pathological case where the scanlist corresponds
         # to a selected top level dataset
         if len(entryNames) == 0:
@@ -115,8 +122,13 @@ class HDF5Stack1D(DataObject.DataObject):
                 if len(scanlist) == 1:
                     if scanlist[0] == ySelection:
                         scanlist = None
-            
-        if scanlist in [None, []]:
+        elif len(entryNames) == 1:
+            # deal with the SOLEIL case of one entry but with different name
+            # in different files
+            USE_JUST_KEYS = True
+        elif scanlist in [None, []]:
+            USE_JUST_KEYS = True
+        if USE_JUST_KEYS:
             #if the scanlist is None, it is assumed we are interested on all
             #the scans containing the selection, not that all the scans
             #contain the selection.
@@ -124,6 +136,7 @@ class HDF5Stack1D(DataObject.DataObject):
             if 0:
                 JUST_KEYS = False
                 #expect same entry names in the files
+                #Unfortunately this does not work for SOLEIL
                 for entry in entryNames:
                     path = "/"+entry + ySelection
                     dirname = posixpath.dirname(path)
@@ -135,19 +148,18 @@ class HDF5Stack1D(DataObject.DataObject):
                         pass
             else:
                 JUST_KEYS = True
-                #expect same structure in the files
+                #expect same structure in the files even if the
+                #names are different (SOLEIL ...)
                 if len(entryNames):
                     i = 0
                     for entry in entryNames:
-                        i += 1
                         path = "/"+entry + ySelection
                         dirname = posixpath.dirname(path)
                         base = posixpath.basename(path)
-                        try:
-                            if base in tmpHdf[dirname].keys():                        
+                        if hasattr(tmpHdf[dirname], "keys"):
+                            i += 1
+                            if base in tmpHdf[dirname].keys():
                                 scanlist.append("1.%d" % i)
-                        except:
-                            pass
                     if not len(scanlist):
                         path = "/" + ySelection
                         dirname = posixpath.dirname(path)
@@ -183,6 +195,10 @@ class HDF5Stack1D(DataObject.DataObject):
             if not nScans:
                 raise IOError("No entry contains the required data")
 
+        if DEBUG:
+            print("Retained number of files = %d" % nFiles)
+            print("Retained number of scans = %d" % nScans)
+
         #Now is to decide the number of mca ...
         #I assume all the scans contain the same number of mca
         if JUST_KEYS:
@@ -202,12 +218,19 @@ class HDF5Stack1D(DataObject.DataObject):
 
         if self.__dtype is None:
             self.__dtype = yDataset.dtype
+            if self.__dtype in [numpy.int16, numpy.uint16]:
+                self.__dtype = numpy.float32
+            elif self.__dtype in [numpy.int32, numpy.uint32]:
+                self.__dtype = numpy.float64
 
         #figure out the shape of the stack
         shape = yDataset.shape
         mcaIndex = selection.get('index', len(shape)-1)
         if mcaIndex == -1:
             mcaIndex = len(shape) - 1
+        if DEBUG:
+            print("mcaIndex = %d" % mcaIndex)
+        considerAsImages = False
         dim0, dim1, mcaDim = self.getDimensions(nFiles, nScans, shape,
                                                 index=mcaIndex)
         try:
@@ -220,17 +243,30 @@ class HDF5Stack1D(DataObject.DataObject):
             else:
                 bytefactor = 8
 
-            neededMegaBytes = nFiles * dim0 * dim1 * mcaDim * bytefactor/(1024*1024.)
-            if (neededMegaBytes > 3000) and (nFiles == 1) and (len(shape) == 3):
+            neededMegaBytes = nFiles * dim0 * dim1 * (mcaDim * bytefactor/(1024*1024.))
+            physicalMemory = PhysicalMemory.getPhysicalMemoryOrNone()
+            if physicalMemory is None:
+                # 5 Gigabytes should be a good compromise
+                physicalMemory = 6000
+            else:
+                physicalMemory /= (1024*1024.)
+            if (neededMegaBytes > (0.95*physicalMemory))\
+               and (nFiles == 1) and (len(shape) == 3):
                 if self.__dtype0 is None:
-                    if (bytefactor == 8) and (neededMegaBytes <6000):
+                    if (bytefactor == 8) and (neededMegaBytes < (2*physicalMemory)):
                         #try reading as float32
                         self.__dtype = numpy.float32
                     else:
                         raise MemoryError("Force dynamic loading")
                 else:
                     raise MemoryError("Force dynamic loading")
-            self.data = numpy.zeros((dim0, dim1, mcaDim), self.__dtype)
+            if (mcaIndex == 0) and ( nFiles == 1) and (nScans == 1):
+                #keep the original arrangement but in memory
+                self.data = numpy.zeros(yDataset.shape, self.__dtype)
+                considerAsImages = True
+            else:
+                # force arrangement as spectra
+                self.data = numpy.zeros((dim0, dim1, mcaDim), self.__dtype)
             DONE = False
         except (MemoryError, ValueError):
             #some versions report ValueError instead of MemoryError
@@ -252,7 +288,7 @@ class HDF5Stack1D(DataObject.DataObject):
                 #what to do if the number of dimensions is only 2?
                 raise
         
-        if not DONE:
+        if (not DONE) and (not considerAsImages):
             self.info["McaIndex"] = 2
             n = 0
 
@@ -263,9 +299,14 @@ class HDF5Stack1D(DataObject.DataObject):
             self.incrProgressBar=0
             for hdf in hdfStack._sourceObjectList:
                 entryNames = list(hdf["/"].keys())
+                goodEntryNames = []
+                for entry in entryNames:
+                    tmpPath = "/" + entry
+                    if hasattr(hdf[tmpPath], "keys"):
+                        goodEntryNames.append(entry)
                 for scan in scanlist:
                     if JUST_KEYS:
-                        entryName = entryNames[int(scan.split(".")[-1])-1]
+                        entryName = goodEntryNames[int(scan.split(".")[-1])-1]
                         path = entryName + ySelection
                         if mSelection is not None:
                             mpath = entryName + mSelection
@@ -433,6 +474,53 @@ class HDF5Stack1D(DataObject.DataObject):
                 if dim0 != 1:
                     self.onProgress(i)
             self.onEnd()
+        elif not DONE:
+            # data into memory but as images
+            self.info["McaIndex"] = mcaIndex            
+            for hdf in hdfStack._sourceObjectList:
+                entryNames = list(hdf["/"].keys())
+                for scan in scanlist:
+                    if JUST_KEYS:
+                        entryName = entryNames[int(scan.split(".")[-1])-1]
+                        path = entryName + ySelection
+                        if mSelection is not None:
+                            mpath = entryName + mSelection
+                            mDataset.shape
+                        if xSelection is not None:
+                            xpath = entryName + xSelection
+                            xDataset = hdf[xpath].value
+                    else:
+                        path = scan + ySelection
+                        if mSelection is not None:
+                            mpath = scan + mSelection
+                            mDataset = hdf[mpath].value
+                        if xSelection is not None:
+                            xpath = scan + xSelection
+                            xDataset = hdf[xpath].value
+                    if mSelection is not None:
+                        nMonitorData = mDataset.size
+                        case = -1
+                        yDatasetShape = yDataset.shape 
+                        if nMonitorData == yDatasetShape[0]:
+                            #as many monitor data as images
+                            mDataset.shape = yDatasetShape[0]
+                            case = 0
+                        elif nMonitorData == (yDatasetShape[1] * yDatasetShape[2]):
+                            #as many monitorData as pixels
+                            case = 1
+                            mDataset.shape = yDatasetShape[1], yDatasetShape[2]
+                        if case == -1:
+                            raise ValueError(\
+                                "I do not know how to handle this monitor data")
+                        if case == 0:
+                            for i in range(yDatasetShape[0]):
+                                self.data[i] = yDataset[i].value / mDataset[i]
+                        elif case == 1:
+                            for i in range(yDataset.shape[0]):
+                                self.data[i] = yDataset[i] / mDataset
+                    else:
+                            for i in range(yDataset.shape[0]):
+                                self.data[i:i+1] = yDataset[i:i+1]
         else:
             self.info["McaIndex"] = mcaIndex
 
