@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2013 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -24,7 +24,7 @@
 # Please contact the ESRF industrial unit (industry@esrf.fr) if this license
 # is a problem for you.
 #############################################################################*/
-__author__ = "V.A. Sole - ESRF BLISS Group"
+__author__ = "V.A. Sole - ESRF Software Group"
 import sys
 import os
 import numpy
@@ -66,15 +66,17 @@ COLORMAPLIST = [spslut.GREYSCALE, spslut.REVERSEGREY, spslut.TEMP,
 
 if QTVERSION > '4.0.0':
     from PyQt4 import Qwt5
+    try:
+        from PyMca import QwtPlotItems
+        OVERLAY_DRAW = True
+    except ImportError:
+        OVERLAY_DRAW = False
 else:
+    OVERLAY_DRAW = False
     import Qwt5
 
-try:
-    from PyMca import QwtPlotItems
-    OVERLAY_DRAW = True
-except ImportError:
-    OVERLAY_DRAW = False
-
+DEFAULT_COLORMAP_INDEX = 2
+DEFAULT_COLORMAP_LOG_FLAG = False
 DEBUG = 0
 
 
@@ -90,13 +92,34 @@ else:
 
 # Uncomment next line if you experience crashes moving the mouse on
 # top of the images
-#USE_PICKER = True
+#USE_PICKER = False
+
+def convertToRowAndColumn(x, y, shape, xScale=None, yScale=None, safe=True):
+    if xScale is None:
+        c = x
+    else:
+        if x < xScale[0]:
+            x = xScale[0]        
+        c = shape[1] *(x - xScale[0]) / (xScale[1] - xScale[0])
+    if yScale is None:
+        r = y
+    else:
+        if y < yScale[0]:
+            y = yScale[0]        
+        r = shape[0] *(y - yScale[0]) / (yScale[1] - yScale[0])
+
+    if safe:
+        c = min(int(c), shape[1] - 1)
+        r = min(int(r), shape[0] - 1)
+    return r, c
 
 class MyPicker(Qwt5.QwtPlotPicker):
     def __init__(self, *var):
         Qwt5.QwtPlotPicker.__init__(self, *var)
         self.__text = Qwt5.QwtText()
         self.data = None
+        self.xScale = None
+        self.yScale = None
 
     if USE_PICKER:
         def trackerText(self, var):
@@ -104,15 +127,13 @@ class MyPicker(Qwt5.QwtPlotPicker):
             if self.data is None:
                 self.__text.setText("%g, %g" % (d.x(), d.y()))
             else:
-                limits = self.data.shape
-                x = round(d.y())
-                y = round(d.x())
-                if x < 0: x = 0
-                if y < 0: y = 0
-                x = min(int(x), limits[0]-1)
-                y = min(int(y), limits[1]-1)
-                z = self.data[x, y]
-                self.__text.setText("%d, %d, %.4g" % (y, x, z))
+                x = d.x()
+                y = d.y()
+                r, c = convertToRowAndColumn(x, y, self.data.shape,
+                                             xScale=self.xScale,
+                                             yScale=self.yScale, safe=True)
+                z = self.data[r, c]
+                self.__text.setText("%.1f, %.1f, %.4g" % (x, y, z))
             return self.__text
     
 class MaskImageWidget(qt.QWidget):
@@ -141,9 +162,13 @@ class MaskImageWidget(qt.QWidget):
         self.__selectionMask = None
         self.__imageData = None
         self.__image = None
+        self._xScale = None
+        self._yScale = None
+
         self.colormap = None
         self.colormapDialog = None
-        self.setDefaultColormap(2, False)
+        self.setDefaultColormap(DEFAULT_COLORMAP_INDEX,
+                                DEFAULT_COLORMAP_LOG_FLAG)
         self.rgbWidget = rgbwidget
 
         self.__imageIconsFlag = imageicons
@@ -168,10 +193,13 @@ class MaskImageWidget(qt.QWidget):
         self._buildConnections()
         self._matplotlibSaveImage = None
 
-        #the overlay items to be drawn
+        # the overlay items to be drawn
         self._overlayItemsDict = {}
-        #the last overlay legend used
+        # the last overlay legend used
         self.__lastOverlayLegend = None
+        self.__lastOverlayWidth = None
+        # the projection mode
+        self.__lineProjectionMode = 'D'
 
     def _build(self, standalonesave, profileselection=False):
         self.mainLayout = qt.QVBoxLayout(self)
@@ -325,6 +353,23 @@ class MaskImageWidget(qt.QWidget):
                      qt.SIGNAL("QtBlissGraphSignal"),
                      self._graphSignal)
 
+    def updateProfileSelectionWindow(self):
+        mode = self.graphWidget.getPickerSelectionMode()
+        if self.__lastOverlayLegend is not None:
+            if mode is None:
+                # remove the overlay if present
+                legend = self.__lastOverlayLegend
+                if legend in self._overlayItemsDict:
+                    self._overlayItemsDict[legend]['item'].detach()
+                    del self._overlayItemsDict[legend]
+            elif self.__lastOverlayWidth is not None:
+                # create a fake signal
+                ddict = {}
+                ddict['event'] = "PolygonWidthChanged"
+                ddict['pixelwidth'] = self.__lastOverlayWidth
+                ddict['mode'] = mode
+                self._polygonSignalSlot(ddict)
+
     def _polygonSignalSlot(self, ddict):
         if DEBUG:
             print("_polygonSignalSLot, event = %s" % ddict['event'])
@@ -377,6 +422,9 @@ class MaskImageWidget(qt.QWidget):
         xdata, ydata, legend, info = curve            
         replot=True
         replace=True
+        idx = numpy.isfinite(ydata)
+        xdata = xdata[idx]
+        ydata = ydata[idx]
         self._profileSelectionWindow.addCurve(xdata, ydata,
                                               legend=legend,
                                               info=info,
@@ -392,6 +440,24 @@ class MaskImageWidget(qt.QWidget):
             title = ""
         return title
 
+    def setLineProjectionMode(self, mode):
+        """
+        Set the line projection mode.
+
+        mode: 1 character string. Allowed options 'D', 'X' 'Y'
+        D - Plot the intensity over the drawn line over as many intervals as pixels over the axis
+            containing the longest projection in pixels.
+        X - Plot the intensity over the drawn line over as many intervals as pixels over the X axis
+        Y - Plot the intensity over the drawn line over as many intervals as pixels over the Y axis
+        """
+        m = mode.upper()
+        if m not in ['D', 'X', 'Y']:
+            raise ValueError("Invalid mode %s. It has to be 'D', 'X' or 'Y'")
+        self.__lineProjectionMode = m
+
+    def getLineProjectionMode(self):
+        return self.__lineProjectionMode
+
     def _getProfileCurve(self, ddict, image=None, overlay=OVERLAY_DRAW):
         if image is None:
             imageData = self.__imageData
@@ -421,9 +487,10 @@ class MaskImageWidget(qt.QWidget):
         shape = imageData.shape
         width = ddict['pixelwidth'] - 1
         if ddict['mode'].upper() in ["HLINE", "HORIZONTAL"]:
+            xLabel = self.getXLabel()
             deltaDistance = 1.0
             if width < 1:
-                row = int(ddict['y'][0])
+                row = int(ddict['row'][0])
                 if row < 0:
                     row = 0
                 if row >= shape[0]:
@@ -439,12 +506,12 @@ class MaskImageWidget(qt.QWidget):
                                          replace=True,
                                          replot=True)
             else:
-                row0 = int(ddict['y'][0]) - 0.5 * width
+                row0 = int(ddict['row'][0]) - 0.5 * width
                 if row0 < 0:
                     row0 = 0
                     row1 = row0 + width
                 else:
-                    row1 = int(ddict['y'][0]) + 0.5 * width
+                    row1 = int(ddict['row'][0]) + 0.5 * width
                 if row1 >= shape[0]:
                     row1 = shape[0] - 1
                     row0 = max(0, row1 - width)
@@ -459,10 +526,13 @@ class MaskImageWidget(qt.QWidget):
                                          replace=True,
                                          replot=True)
             xdata  = numpy.arange(shape[1]).astype(numpy.float)
+            if self._xScale is not None:
+                xdata = self._xScale[0] + xdata * (self._xScale[1] - self._xScale[0]) / float(shape[1])
         elif ddict['mode'].upper() in ["VLINE", "VERTICAL"]:
+            xLabel = self.getYLabel()
             deltaDistance = 1.0
             if width < 1:
-                column = int(ddict['x'][0])
+                column = int(ddict['column'][0])
                 if column < 0:
                     column = 0
                 if column >= shape[1]:
@@ -478,12 +548,12 @@ class MaskImageWidget(qt.QWidget):
                                          replace=True,
                                          replot=True)
             else:
-                col0 = int(ddict['x'][0]) - 0.5 * width
+                col0 = int(ddict['column'][0]) - 0.5 * width
                 if col0 < 0:
                     col0 = 0
                     col1 = col0 + width
                 else:
-                    col1 = int(ddict['x'][0]) + 0.5 * width
+                    col1 = int(ddict['column'][0]) + 0.5 * width
                 if col1 >= shape[1]:
                     col1 = shape[1] - 1
                     col0 = max(0, col1 - width)
@@ -498,25 +568,35 @@ class MaskImageWidget(qt.QWidget):
                                          replace=True,
                                          replot=True)
             xdata  = numpy.arange(shape[0]).astype(numpy.float)
+            if self._yScale is not None:
+                xdata = self._yScale[0] + xdata * (self._yScale[1] - self._yScale[0]) / float(shape[0])
         elif ddict['mode'].upper() in ["LINE"]:
-            if len(ddict['x']) == 1:
+            if len(ddict['column']) == 1:
                 #only one point given
                 return
             #the coordinates of the reference points
             x0 = numpy.arange(float(shape[0]))
             y0 = numpy.arange(float(shape[1]))
             #get the interpolation points
-            col0, col1 = [int(x) for x in ddict['x']]
-            row0, row1 = [int(x) for x in ddict['y']]
+            col0, col1 = [int(x) for x in ddict['column']]
+            row0, row1 = [int(x) for x in ddict['row']]
             deltaCol = abs(col0 - col1)
             deltaRow = abs(row0 - row1)
-            if deltaCol > deltaRow:
-                npoints = deltaCol+1
-            else:    
-                npoints = deltaRow+1
-            #get the abscisa in distance units
-            deltaDistance = numpy.sqrt(float(deltaCol)*deltaCol +
-                                       float(deltaRow)*deltaRow)/(npoints-1.0)
+            if self.__lineProjectionMode == 'D':
+                if deltaCol >= deltaRow:
+                    npoints = deltaCol + 1
+                else:    
+                    npoints = deltaRow + 1
+            elif self.__lineProjectionMode == 'X':
+                npoints = deltaCol + 1
+            else:
+                npoints = deltaRow + 1
+            if npoints == 1:
+                #all points are the same
+                if DEBUG:
+                    print("START AND END POINT ARE THE SAME!!")
+                return
+
             if width < 1:
                 x = numpy.zeros((npoints, 2), numpy.float)
                 x[:, 0] = numpy.linspace(row0, row1, npoints)
@@ -536,17 +616,17 @@ class MaskImageWidget(qt.QWidget):
                                          replot=True)
             elif deltaCol == 0:
                 #vertical line
-                col0 = int(ddict['x'][0]) - 0.5 * width
+                col0 = int(ddict['column'][0]) - 0.5 * width
                 if col0 < 0:
                     col0 = 0
                     col1 = col0 + width
                 else:
-                    col1 = int(ddict['x'][0]) + 0.5 * width
+                    col1 = int(ddict['column'][0]) + 0.5 * width
                 if col1 >= shape[1]:
                     col1 = shape[1] - 1
                     col0 = max(0, col1 - width)
-                row0 = int(ddict['y'][0])
-                row1 = int(ddict['y'][1])
+                row0 = int(ddict['row'][0])
+                row1 = int(ddict['row'][1])
                 if row0 > row1:
                     tmp = row0
                     row0 = row1
@@ -569,17 +649,17 @@ class MaskImageWidget(qt.QWidget):
                                          replot=True)
             elif deltaRow == 0:
                 #horizontal line
-                row0 = int(ddict['y'][0]) - 0.5 * width
+                row0 = int(ddict['row'][0]) - 0.5 * width
                 if row0 < 0:
                     row0 = 0
                     row1 = row0 + width
                 else:
-                    row1 = int(ddict['y'][0]) + 0.5 * width
+                    row1 = int(ddict['row'][0]) + 0.5 * width
                 if row1 >= shape[0]:
                     row1 = shape[0] - 1
                     row0 = max(0, row1 - width)
-                col0 = int(ddict['x'][0])
-                col1 = int(ddict['x'][1])
+                col0 = int(ddict['column'][0])
+                col1 = int(ddict['column'][1])
                 if col0 > col1:
                     tmp = col0
                     col0 = col1
@@ -607,10 +687,6 @@ class MaskImageWidget(qt.QWidget):
                 m = (row1 - row0) / float((col1 - col0))
                 b = row0 - m * col0
                 alpha = numpy.arctan(m)
-                if deltaRow > deltaCol:
-                    npoints = deltaRow + 1
-                else:
-                    npoints = deltaCol + 1
                 #imagine the following sequence
                 # - change origin to the first point
                 # - clock-wise rotation to bring the line on the x axis of a new system
@@ -749,19 +825,35 @@ class MaskImageWidget(qt.QWidget):
                                          info=ddict,
                                          replace=True,
                                          replot=True)
+            if self.__lineProjectionMode == 'X':
+                xLabel = self.getXLabel()
+                xdata += col0
+                if self._xScale is not None:
+                    xdata = self._xScale[0] + xdata * (self._xScale[1] - self._xScale[0]) / float(shape[1])
+            elif self.__lineProjectionMode == 'Y':
+                xLabel = self.getYLabel()
+                xdata += row0
+                if self._xScale is not None:
+                    xdata = self._yScale[0] + xdata * (self._yScale[1] - self._yScale[0]) / float(shape[0])
+            else:
+                xLabel = "Distance"
+                if self._xScale is not None:
+                    deltaCol *= (self._xScale[1] - self._xScale[0])/float(shape[1])
+                    deltaRow *= (self._yScale[1] - self._yScale[0])/float(shape[0])
+                #get the abscisa in distance units
+                deltaDistance = numpy.sqrt(float(deltaCol) * deltaCol +
+                                    float(deltaRow) * deltaRow)/(npoints-1.0)
+                xdata *= deltaDistance
         else:
             if DEBUG:
                 print("Mode %s not supported yet" % ddict['mode'])
             return
+
+        self.__lastOverlayWidth = width
         info = {}
+        info['xlabel'] = xLabel
         info['ylabel'] = "Z"
-        if 0:
-            info['xlabel'] = "Point"
-            return xdata, ydata, legend, info
-        else:
-            #users prefer to work with distance
-            info['xlabel'] = "Distance"
-            return xdata * deltaDistance, ydata, legend, info
+        return xdata, ydata, legend, info
 
     def _profileSelectionSlot(self, ddict):
         if DEBUG:
@@ -834,9 +926,24 @@ class MaskImageWidget(qt.QWidget):
                 for key in delKeys:
                     del self._overlayItemsDict[name][key]
                 del self._overlayItemsDict[name]
-        overlayItem.setData(x, y)
-        self._overlayItemsDict[legend]['x'] = x
-        self._overlayItemsDict[legend]['y'] = y
+        #the type of x can be list or array
+        shape = self.__imageData.shape
+        if self._xScale is None:
+            xList = x
+        else:
+            xList = []
+            for i in x:
+                xList.append(self._xScale[0] + i * (self._xScale[1] - self._xScale[0])/float(shape[1]))
+
+        if self._yScale is None:
+            yList = y
+        else:
+            yList = []
+            for i in y:
+                yList.append(self._yScale[0] + i * (self._yScale[1] - self._yScale[0])/float(shape[0]))
+        overlayItem.setData(xList, yList)
+        self._overlayItemsDict[legend]['x'] = xList
+        self._overlayItemsDict[legend]['y'] = yList
         self._overlayItemsDict[legend]['info'] = info
         if replot:
             self.graphWidget.graph.replot()
@@ -871,6 +978,18 @@ class MaskImageWidget(qt.QWidget):
         self._y1AxisInverted = value
         self.graphWidget.graph.setY1AxisInverted(self._y1AxisInverted)
 
+    def setXLabel(self, label="Column"):
+        return self.graphWidget.setXLabel(label)
+
+    def setYLabel(self, label="Row"):
+        return self.graphWidget.setYLabel(label)
+
+    def getXLabel(self):
+        return self.graphWidget.getXLabel()
+
+    def getYLabel(self):
+        return self.graphWidget.getYLabel()
+
     def buildAndConnectImageButtonBox(self, replace=True):
         # The IMAGE selection
         self.imageButtonBox = qt.QWidget(self)
@@ -1099,7 +1218,9 @@ class MaskImageWidget(qt.QWidget):
     def setSelectionMask(self, mask, plot=True):
         if mask is not None:
             if self.__imageData is not None:
-                mask *=  numpy.isfinite(self.__imageData)
+                # this operation will be made when retrieving the mask
+                #mask *= numpy.isfinite(self.__imageData)
+                pass
         self.__selectionMask = mask
         if plot:
             self.plotImage(update=False)
@@ -1108,11 +1229,15 @@ class MaskImageWidget(qt.QWidget):
         if self.__imageData is None:
             return None
         if self.__selectionMask is None:
-            return numpy.zeros(self.__imageData.shape, numpy.uint8)
-        return self.__selectionMask
+             return numpy.zeros(self.__imageData.shape, numpy.uint8) *\
+                    numpy.isfinite(self.__imageData)
+        return self.__selectionMask *\
+                    numpy.isfinite(self.__imageData)
 
-    def setImageData(self, data, clearmask=False):
+    def setImageData(self, data, clearmask=False, xScale=None, yScale=None):
         self.__image = None
+        self._xScale = xScale
+        self._yScale = yScale
         if data is None:
             self.__imageData = data
             self.__selectionMask = None
@@ -1174,6 +1299,8 @@ class MaskImageWidget(qt.QWidget):
         else:
             self.__imageData = data
             self.__imageData.shape = height, width
+        self._xScale = None
+        self._yScale = None
         self.__pixmap0 = pixmap
         if clearmask:
             self.__selectionMask = None
@@ -1183,12 +1310,16 @@ class MaskImageWidget(qt.QWidget):
         if self.__imageData is None:
             self.graphWidget.graph.clear()
             self.graphWidget.picker.data = None
+            self.graphWidget.picker.xScale = None
+            self.graphWidget.picker.yScale = None
             return
 
         if update:
             self.getPixmapFromData()
             self.__pixmap0 = self.__pixmap.copy()
             self.graphWidget.picker.data = self.__imageData
+            self.graphWidget.picker.xScale = self._xScale
+            self.graphWidget.picker.yScale = self._yScale
             if self.colormap is None:
                 if self.__defaultColormap < 2:
                     self.graphWidget.picker.setTrackerPen(qt.QPen(qt.Qt.green))
@@ -1206,7 +1337,9 @@ class MaskImageWidget(qt.QWidget):
         self.graphWidget.graph.pixmapPlot(self.__pixmap.tostring(),
             (self.__imageData.shape[1], self.__imageData.shape[0]),
                                         xmirror = 0,
-                                        ymirror = not self._y1AxisInverted)
+                                        ymirror = not self._y1AxisInverted,
+                                        xScale = self._xScale,
+                                        yScale = self._yScale)
 
         if not self.graphWidget.graph.yAutoScale:
             self.graphWidget.graph.setY1AxisLimits(ylimits[0], ylimits[1],
@@ -1215,6 +1348,7 @@ class MaskImageWidget(qt.QWidget):
             self.graphWidget.graph.setX1AxisLimits(xlimits[0], xlimits[1],
                                                    replot=False)
         self.graphWidget.graph.replot()
+        self.updateProfileSelectionWindow()
 
     def getPixmapFromData(self):
         colormap = self.colormap
@@ -1222,25 +1356,44 @@ class MaskImageWidget(qt.QWidget):
             self.__pixmap = self.__pixmap0.copy()
             return
 
-        tmpData = numpy.isfinite(self.__imageData)
-        goodData = tmpData.min()
+        if hasattr(self.__imageData, 'mask'):
+            data = self.__imageData.data
+        else:
+            data = self.__imageData
+
+        finiteData = numpy.isfinite(data)
+        goodData = finiteData.min()
         
         if self.colormapDialog is not None:
             minData = self.colormapDialog.dataMin
             maxData = self.colormapDialog.dataMax
         else:
             if goodData:
-                minData = self.__imageData.min()
-                maxData = self.__imageData.max()
+                minData = data.min()
+                maxData = data.max()
             else:
-                tmpData = self.__imageData[tmpData]
-                minData = tmpData.min()
-                maxData = tmpData.max()
-        tmpData = None
-                
+                tmpData = data[finiteData]
+                if tmpData.size > 0:
+                    minData = tmpData.min()
+                    maxData = tmpData.max()
+                else:
+                    minData = None
+                    maxData = None
+                tmpData = None
         if colormap is None:
-            (self.__pixmap,size,minmax)= spslut.transform(\
-                                self.__imageData,
+            if minData is None:
+                (self.__pixmap,size,minmax)= spslut.transform(\
+                                data,
+                                (1,0),
+                                (self.__defaultColormapType,3.0),
+                                "BGRX",
+                                self.__defaultColormap,
+                                1,
+                                (0, 1),
+                                (0, 255), 1)
+            else:
+                (self.__pixmap,size,minmax)= spslut.transform(\
+                                data,
                                 (1,0),
                                 (self.__defaultColormapType,3.0),
                                 "BGRX",
@@ -1253,7 +1406,7 @@ class MaskImageWidget(qt.QWidget):
                 colormap.append(spslut.LINEAR)
             if goodData:
                 (self.__pixmap,size,minmax)= spslut.transform(\
-                                self.__imageData,
+                                data,
                                 (1,0),
                                 (colormap[6],3.0),
                                 "BGRX",
@@ -1263,8 +1416,19 @@ class MaskImageWidget(qt.QWidget):
                                 (0,255), 1)                
             elif colormap[1]:
                 #autoscale
-                (self.__pixmap,size,minmax)= spslut.transform(\
-                                self.__imageData,
+                if minData is None:
+                    (self.__pixmap,size,minmax)= spslut.transform(\
+                                data,
+                                (1,0),
+                                (self.__defaultColormapType,3.0),
+                                "BGRX",
+                                self.__defaultColormap,
+                                1,
+                                (0, 1),
+                                (0, 255), 1)
+                else:
+                    (self.__pixmap,size,minmax)= spslut.transform(\
+                                data,
                                 (1,0),
                                 (colormap[6],3.0),
                                 "BGRX",
@@ -1274,7 +1438,7 @@ class MaskImageWidget(qt.QWidget):
                                 (0,255), 1)
             else:
                 (self.__pixmap,size,minmax)= spslut.transform(\
-                                self.__imageData,
+                                data,
                                 (1,0),
                                 (colormap[6],3.0),
                                 "BGRX",
@@ -1285,9 +1449,10 @@ class MaskImageWidget(qt.QWidget):
 
         self.__pixmap = self.__pixmap.astype(numpy.ubyte)
 
-        self.__pixmap.shape = [self.__imageData.shape[0],
-                                    self.__imageData.shape[1],
-                                    4]
+        self.__pixmap.shape = [data.shape[0], data.shape[1], 4]
+
+        if not goodData:
+            self.__pixmap[finiteData < 1] = 255
 
     def __applyMaskToImage(self):
         if self.__selectionMask is None:
@@ -1368,6 +1533,8 @@ class MaskImageWidget(qt.QWidget):
             return
         if self.colormapDialog is None:
             self.__initColormapDialog()
+            if self.colormapDialog is None:
+                return
         if self.colormapDialog.isHidden():
             self.colormapDialog.show()
         if QTVERSION < '4.0.0':
@@ -1378,9 +1545,14 @@ class MaskImageWidget(qt.QWidget):
 
     def __initColormapDialog(self):
         goodData = self.__imageData[numpy.isfinite(self.__imageData)]
-        maxData = goodData.max()
-        minData = goodData.min()
-        self.colormapDialog = ColormapDialog.ColormapDialog()
+        if goodData.size > 0:
+            maxData = goodData.max()
+            minData = goodData.min()
+        else:
+            qt.QMessageBox.critical(self,"No Data",
+                "Image data does not contain any real value")
+            return
+        self.colormapDialog = ColormapDialog.ColormapDialog(self)
         colormapIndex = self.__defaultColormap
         if colormapIndex == 1:
             colormapIndex = 0
@@ -1467,8 +1639,8 @@ class MaskImageWidget(qt.QWidget):
                                                             image=None)
         title = "Matplotlib " + self.getGraphTitle()
         self._matplotlibSaveImage.setWindowTitle(title)        
+        ddict = self._matplotlibSaveImage.getParameters()
         if self.colormap is not None:
-            ddict = self._matplotlibSaveImage.getParameters()
             colormapType = ddict['linlogcolormap']
             try:
                 colormapIndex, autoscale, vmin, vmax,\
@@ -1486,7 +1658,26 @@ class MaskImageWidget(qt.QWidget):
             else:
                 ddict['valuemin'] = 0
                 ddict['valuemax'] = 0
-            self._matplotlibSaveImage.setParameters(ddict)
+            
+        #this sets the actual dimensions
+        if self._xScale is not None:
+            ddict['xorigin'] = self._xScale[0]
+            ddict['xpixelsize'] = (self._xScale[1] - self._xScale[0])/\
+                                      float(imageData.shape[1])
+        if self._yScale is not None:
+            ddict['yorigin'] = self._yScale[0]
+            ddict['ypixelsize'] = (self._yScale[1] - self._yScale[0])/\
+                                      float(imageData.shape[0])
+        ddict['xlabel'] = self.getXLabel()
+        ddict['ylabel'] = self.getYLabel()
+        limits = self.graphWidget.graph.getX1AxisLimits()
+        ddict['zoomxmin'] = limits[0]
+        ddict['zoomxmax'] = limits[1]
+        limits = self.graphWidget.graph.getY1AxisLimits()
+        ddict['zoomymin'] = limits[0] 
+        ddict['zoomymax'] = limits[1]
+        
+        self._matplotlibSaveImage.setParameters(ddict)
         self._matplotlibSaveImage.setImageData(imageData)
         self._matplotlibSaveImage.show()
         self._matplotlibSaveImage.raise_()
@@ -1501,22 +1692,33 @@ class MaskImageWidget(qt.QWidget):
         if self.__imageData is None:
             return
         if ddict['event'] == "MouseSelection":
-            if ddict['xmin'] < ddict['xmax']:
-                xmin = ddict['xmin']
-                xmax = ddict['xmax']
+            if ddict['column_min'] < ddict['column_max']:
+                xmin = ddict['column_min']
+                xmax = ddict['column_max']
             else:
-                xmin = ddict['xmax']
-                xmax = ddict['xmin']
-            if ddict['ymin'] < ddict['ymax']:
-                ymin = ddict['ymin']
-                ymax = ddict['ymax']
+                xmin = ddict['column_max']
+                xmax = ddict['column_min']
+            if ddict['row_min'] < ddict['row_max']:
+                ymin = ddict['row_min']
+                ymax = ddict['row_max']
             else:
-                ymin = ddict['ymax']
-                ymax = ddict['ymin']
+                ymin = ddict['row_max']
+                ymax = ddict['row_min']
+            """
+            if not (self._xScale is None and self._yScale is None):
+                ymin, xmin = convertToRowAndColumn(xmin, ymin, self.__imageData.shape,
+                                                  xScale=self._xScale,
+                                                  yScale=self._yScale,
+                                                  safe=True)
+                ymax, xmax = convertToRowAndColumn(xmax, ymax, self.__imageData.shape,
+                                                  xScale=self._xScale,
+                                                  yScale=self._yScale,
+                                                  safe=True)
+            """
             i1 = max(int(round(xmin)), 0)
-            i2 = min(abs(int(round(xmax)))+1, self.__imageData.shape[1])
+            i2 = min(abs(int(round(xmax))) + 1, self.__imageData.shape[1])
             j1 = max(int(round(ymin)),0)
-            j2 = min(abs(int(round(ymax)))+1,self.__imageData.shape[0])
+            j2 = min(abs(int(round(ymax))) + 1, self.__imageData.shape[0])
             if self.__selectionMask is None:
                 self.__selectionMask = numpy.zeros(self.__imageData.shape,
                                      numpy.uint8)
@@ -1532,19 +1734,35 @@ class MaskImageWidget(qt.QWidget):
                 #if follow mouse is not activated
                 #it only enters here when the mouse is pressed.
                 #Therefore is perfect for "brush" selections.
+                """
+                if not (self._xScale is None and self._yScale is None):
+                    y, x = convertToRowAndColumn(ddict['x'], ddict['y'], self.__imageData.shape,
+                                                      xScale=self._xScale,
+                                                      yScale=self._yScale,
+                                                      safe=True)
+                else:
+                    x = ddict['x']
+                    y = ddict['y']
+                """
+                y = ddict['row']
+                x = ddict['column']
                 width = self.__brushWidth   #in (row, column) units
                 r = self.__imageData.shape[0]
                 c = self.__imageData.shape[1]
-                xmin = max((ddict['x']-0.5*width), 0)
-                xmax = min((ddict['x']+0.5*width), c)
-                ymin = max((ddict['y']-0.5*width), 0)
-                ymax = min((ddict['y']+0.5*width), r)
+
+                xmin = max((x-0.5*width), 0)
+                xmax = min((x+0.5*width), c)
+                ymin = max((y-0.5*width), 0)
+                ymax = min((y+0.5*width), r)
+                
                 i1 = min(int(round(xmin)), c-1)
                 i2 = min(int(round(xmax)), c)
                 j1 = min(int(round(ymin)),r-1)
                 j2 = min(int(round(ymax)), r)
-                if i1 == i2: i2 = i1+1
-                if j1 == j2: j2 = j1+1
+                if i1 == i2:
+                    i2 = i1+1
+                if j1 == j2:
+                    j2 = j1+1
                 if self.__selectionMask is None:
                     self.__selectionMask = numpy.zeros(self.__imageData.shape,
                                      numpy.uint8)
@@ -1610,7 +1828,8 @@ class MaskImageWidget(qt.QWidget):
         filedialog.selectFilter(self._saveFilter)
         filedialog.setDirectory(initdir)
         ret = filedialog.exec_()
-        if not ret: return ""
+        if not ret:
+            return ""
         filename = filedialog.selectedFiles()[0]
         if len(filename):
             filename = qt.safe_str(filename)
@@ -1745,10 +1964,10 @@ def test():
             container.setQImage(image, 200, 200)
     else:
         container = MaskImageWidget(profileselection=True)
-        data = numpy.arange(40000).astype(numpy.int32)
-        data.shape = 200, 200
+        data = numpy.arange(400 * 200).astype(numpy.int32)
+        data.shape = 400, 200
         #data = numpy.eye(200)
-        container.setImageData(data)
+        container.setImageData(data, xScale=(200, 800), yScale=(400., 800.))
         #data.shape = 100, 400
         #container.setImageData(None)
         #container.setImageData(data)
