@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2013 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -398,7 +398,12 @@ def numpyPCA(stack, ncomponents=10, binning=None, **kw):
     """
     This is a covariance method using numpy
     """
+    if DEBUG:
+        print("PCAModule.numpyPCA called")
+    if hasattr(stack, "info"):
+        index = stack.info.get('McaIndex', -1)
     return PCATools.numpyPCA(stack,
+                             index=index,
                              ncomponents=ncomponents,
                              binning=binning,
                              **kw)
