@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2013 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -25,6 +25,7 @@
 # is a problem for you.
 #############################################################################*/
 __author__ = "V.A. Sole - ESRF Data Analysis"
+import sys
 import numpy
 import numpy.linalg
 try:
@@ -36,7 +37,6 @@ except ImportError:
 
 DEBUG = 0
 
-
 def getCovarianceMatrix(stack,
                         index=-1,
                         binning=None,
@@ -84,7 +84,7 @@ def getCovarianceMatrix(stack,
     #
 
     #the starting number of channels or of images
-    N = oldShape[index]
+    N = oldShape[actualIndex]
 
     # our binning (better said sampling) is spectral, in order not to
     # affect the spatial resolution
@@ -97,7 +97,7 @@ def getCovarianceMatrix(stack,
     if spatial_mask is not None:
         cleanMask = spatial_mask[:].reshape(nPixels)
         usedPixels = cleanMask.sum()
-        badMask = ~spatial_mask
+        badMask = numpy.array(spatial_mask < 1, dtype=cleanMask.dtype)
         badMask.shape = nPixels
     else:
         cleanMask = None
@@ -110,6 +110,8 @@ def getCovarianceMatrix(stack,
     eigenvectorLength = nChannels
 
     if (not force)and isinstance(data, numpy.ndarray):
+        if DEBUG:
+            print("Memory consuming calculation")
         #make a direct calculation (memory cosuming)
         #take a view to the data
         dataView = data[:]
@@ -163,7 +165,25 @@ def getCovarianceMatrix(stack,
         data = None
         raise
 
-    if index in [0]:
+    #workaround a problem with h5py
+    try:
+        if actualIndex in [0]:
+            testException = data[0:1]
+        else:
+            if len(data.shape) == 2:
+                testException = data[0:1,-1]
+            elif len(data.shape) == 3:
+                testException = data[0:1,0:1,-1]
+    except AttributeError:
+        txt = "%s" % type(data)
+        if 'h5py' in txt:
+            print("Implementing h5py workaround")
+            import h5py
+            data = h5py.Dataset(data.id)
+        else:
+            raise
+
+    if actualIndex in [0]:
         #divider is used to decide the fraction of images to keep in memory
         #in order to limit file access on dynamically loaded data.
         #Since two chunks of the same size are used, the amount of memory
@@ -470,6 +490,11 @@ def getCovarianceMatrix(stack,
 
 def numpyPCA(stack, index=-1, ncomponents=10, binning=None,
                 center=True, scale=False, mask=None, **kw):
+    if DEBUG:
+        print("PCATools.numpyPCA")
+        print("index = %d" % index)
+        print("center = %s" % center)
+        print("scale = %s" % scale)
     #recover the actual data to work with
     if hasattr(stack, "info") and hasattr(stack, "data"):
         #we are dealing with a PyMca data object
@@ -477,6 +502,7 @@ def numpyPCA(stack, index=-1, ncomponents=10, binning=None,
     else:
         data = stack
 
+    force = kw.get("force", True)
     oldShape = data.shape
     if index not in [0, -1, len(oldShape) - 1]:
         data = None
@@ -488,6 +514,24 @@ def numpyPCA(stack, index=-1, ncomponents=10, binning=None,
     else:
         actualIndex = index
 
+    #workaround a problem with h5py
+    try:
+        if actualIndex in [0]:
+            testException = data[0:1]
+        else:
+            if len(data.shape) == 2:
+                testException = data[0:1,-1]
+            elif len(data.shape) == 3:
+                testException = data[0:1,0:1,-1]
+    except AttributeError:
+        txt = "%s" % type(data)
+        if 'h5py' in txt:
+            print("Implementing h5py workaround")
+            import h5py
+            data = h5py.Dataset(data.id)
+        else:
+            raise
+
     #the number of spatial pixels
     nPixels = 1
     for i in range(len(oldShape)):
@@ -509,7 +553,7 @@ def numpyPCA(stack, index=-1, ncomponents=10, binning=None,
     cov, avgSpectrum, calculatedPixels = getCovarianceMatrix(stack,
                                                              index=index,
                                                              binning=binning,
-                                                             force=False,
+                                                             force=force,
                                                              center=center,
                                                              spatial_mask=mask)
 
@@ -542,16 +586,19 @@ def numpyPCA(stack, index=-1, ncomponents=10, binning=None,
         a = [(evalues[i], i) for i in range(len(evalues))]
         a.sort()
         a.reverse()
+        totalExplainedVariance = 0.0
         for i0 in range(ncomponents):
             i = a[i0][1]
             eigenvalues[i0] = evalues[i]
             if normalizeToUnitStandardDeviation:
-                print("PC%02d  Explained variance %.5f %% " %\
-                                (i0 + 1, 100. * evalues[i] / totalVariance.shape[0]))
+                partialExplainedVariance = 100. * evalues[i] / totalVariance.shape[0]
             else:
-                print("PC%02d  Explained variance %.5f %% " %\
-                                (i0 + 1, 100. * evalues[i] / totalVariance.sum()))
+                partialExplainedVariance = 100. * evalues[i] / totalVariance.sum()
+            print("PC%02d  Explained variance %.5f %% " %\
+                                        (i0 + 1, partialExplainedVariance))
+            totalExplainedVariance += partialExplainedVariance
             eigenvectors[i0, :] = evectors[:, i]
+        print("Total explained variance = %.2f %% " % totalExplainedVariance)
     else:
         idx = numpy.argsort(evalues)
         eigenvalues[:]  = evalues[idx]
@@ -708,13 +755,11 @@ def test():
 
 
 if __name__ == "__main__":
-    import sys
     test()
     sys.exit(0)
     from PyMca import EDFStack
     from PyMca import EdfFile
     import os
-    import sys
     import time
     #inputfile = ".\PierreSue\CH1777\G4-Sb\G4_mca_0012_0000_0000.edf"
     inputfile = "D:\DATA\COTTE\ch09\ch09__mca_0005_0000_0000.edf"    
