@@ -4,26 +4,18 @@
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
 #
-# This toolkit is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by the Free
-# Software Foundation; either version 2 of the License, or (at your option)
+# This file is free software; you can redistribute it and/or modify it
+# under the terms of the GNU Lesser General Public License as published by the
+# Free Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
 #
-# PyMca is distributed in the hope that it will be useful, but WITHOUT ANY
+# This file is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 # details.
 #
-# You should have received a copy of the GNU General Public License along with
-# PyMca; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-#
-# PyMca follows the dual licensing model of Riverbank's PyQt and cannot be
-# used as a free plugin for a non-free program.
-#
-# Please contact the ESRF industrial unit (industry@esrf.fr) if this license
-# is a problem for you.
 #############################################################################*/
+__author__ = "V.A. Sole - ESRF Data Analysis"
 __doc__= "Interface to the PyMca EPDL97 description" 
 import os
 import sys
@@ -162,6 +154,10 @@ def _initializeElement(element):
             EPDL97_DICT[element]['EPDL97']['compton'] = data[i, :]
             EPDL97_DICT[element]['EPDL97']['compton'].shape = -1
             continue
+        if 'allother' in label:
+            EPDL97_DICT[element]['EPDL97']['all other'] = data[i, :]
+            EPDL97_DICT[element]['EPDL97']['all other'].shape = -1
+            continue
         label = label.replace(" ","").split("(")[0]
         if 'energy' in label:
             EPDL97_DICT[element]['EPDL97']['energy'] = data[i, :]
@@ -183,29 +179,40 @@ def _initializeElement(element):
             EPDL97_DICT[element]['EPDL97'][label.upper()] = data[i, :]
             EPDL97_DICT[element]['EPDL97'][label.upper()].shape = -1
             continue
-    EPDL97_DICT[element]['EPDL97']['pair'] = 0.0 * EPDL97_DICT[element]['EPDL97']['energy']
-    EPDL97_DICT[element]['EPDL97']['total'] =\
-            EPDL97_DICT[element]['EPDL97']['coherent']+\
-            EPDL97_DICT[element]['EPDL97']['compton']+\
-            EPDL97_DICT[element]['EPDL97']['pair']+\
-            EPDL97_DICT[element]['EPDL97']['photo']
-    EPDL97_DICT[element]['EPDL97']['all other']=1 *\
-            EPDL97_DICT[element]['EPDL97']['photo']
-    atomic_shells = ['K', 'L1', 'L2', 'L3', 'M1', 'M2', 'M3', 'M4', 'M5']
-    for key in atomic_shells:
-        EPDL97_DICT[element]['EPDL97']['all other']-=\
-                EPDL97_DICT[element]['EPDL97'][key]
+    EPDL97_DICT[element]['EPDL97']['pair'] = 0.0 *\
+                                             EPDL97_DICT[element]['EPDL97']['energy']
+    EPDL97_DICT[element]['EPDL97']['photo'] = \
+            EPDL97_DICT[element]['EPDL97']['total'] -\
+            EPDL97_DICT[element]['EPDL97']['compton']-\
+            EPDL97_DICT[element]['EPDL97']['coherent']-\
+            EPDL97_DICT[element]['EPDL97']['pair']
+
+    atomic_shells = ['M5', 'M4', 'M3', 'M2', 'M1', 'L3', 'L2', 'L1', 'K']
+
+    # with the new (short) version of the cross-sections file, "all other" contains all
+    # shells above the M5. Nevertheless, we calculate it
+    if scan_index > 17:
+        idx = EPDL97_DICT[element]['EPDL97']['all other'] > 0.0
+        delta = 0.0
+        for key in atomic_shells:
+            delta += EPDL97_DICT[element]['EPDL97'][key]
+        EPDL97_DICT[element]['EPDL97']['all other'] =\
+                        (EPDL97_DICT[element]['EPDL97']['photo'] - delta) * idx
+    else:
+        EPDL97_DICT[element]['EPDL97']['all other'] = 0.0 * \
+                        EPDL97_DICT[element]['EPDL97']['photo']
 
     #take care of rounding problems
-    EPDL97_DICT[element]['EPDL97']['all other']\
-        [EPDL97_DICT[element]['EPDL97']['all other'] < 0.0] = 0.0
-
+    idx = EPDL97_DICT[element]['EPDL97']['all other'] < 0.0
+    EPDL97_DICT[element]['EPDL97']['all other'][idx] = 0.0
+    
+    
 def getElementCrossSections(element, energy=None, forced_shells=None):
     """
-    getCrossSections(element, energy, excited_shells=None)
+    getElementCrossSections(element, energy, forced_shells=None)
     Returns total and partial cross sections of element at the specified
-    energies. If excited_shells are not specified, it uses the internal
-    binding energies of EPDL97 for all shells. If excited_shells is specified,
+    energies. If forced_shells are not specified, it uses the internal
+    binding energies of EPDL97 for all shells. If forced_shells is specified,
     it enforces excitation of the relevant shells via log-log extrapolation
     if needed.
     """
@@ -240,42 +247,49 @@ def getElementCrossSections(element, energy=None, forced_shells=None):
     ddict['pair']       = 0.0 * energy
     ddict['all other']  = 0.0 * energy
     ddict['total']      = 0.0 * energy
-    atomic_shells = ['K', 'L1', 'L2', 'L3', 'M1', 'M2', 'M3', 'M4', 'M5']
+    atomic_shells = ['M5', 'M4', 'M3', 'M2', 'M1', 'L3', 'L2', 'L1', 'K']
     for key in atomic_shells:
         ddict[key] = 0.0 * energy
 
     #find interpolation point
-    for i in range(len(energy)):
+    len_energy = len(energy)
+    for i in range(len_energy):
         x = energy[i]
         if x > wdata['energy'][-2]:
             #take last value or extrapolate?
             print("Warning: Extrapolating data at the end")
             j1 = len(wdata['energy']) - 1 
             j0 = j1 - 1
-        elif x < wdata['energy'][0]:
+        elif x <= wdata['energy'][0]:
             #take first value or extrapolate?
             print("Warning: Extrapolating data at the beginning")
             j1 = 1 
             j0 = 0
         else:
-            j0 = numpy.max(numpy.nonzero(wdata['energy'] <= x), axis=1)
+            j0 = numpy.max(numpy.nonzero(wdata['energy'] < x), axis=1)
             j1 = j0 + 1
         x0 = wdata['energy'][j0]
         x1 = wdata['energy'][j1]
+        if x == x1:
+            if (j1 + 1 ) < len(wdata['energy']):
+                if x1 == wdata['energy'][j1 + 1]:
+                    j0 = j1
+                    j1 += 1
+                    x0 = wdata['energy'][j0]
+                    x1 = wdata['energy'][j1]
 
         #coherent and incoherent
-        for key in ['coherent', 'compton', 'all other']:
-            y0 = wdata[key][j0]
-            y1 = wdata[key][j1]
-            #if key == 'all other':
-            #    print "energy = ", x
-            #    print "x0 = ", x0
-            #    print "x1 = ", x1
-            #    print 1, y0
-            #    print 2, y1
-            if (y0 > 0) and (y1 > 0):
-                ddict[key][i] = exp((log(y0) * log(x1/x) +\
-                                 log(y1) * log(x/x0))/log(x1/x0))
+        for key in ['coherent', 'compton', 'pair', 'all other']:
+            if (j0 == j1) or ((x1 - x0) < 5.E-10) or ((x1 - x) < 5.E-10) :
+                ddict[key][i] =  wdata[key][j1]
+            else:
+                y0 = wdata[key][j0]
+                y1 = wdata[key][j1]
+                if (y0 > 0) and (y1 > 0):
+                    ddict[key][i] = exp((log(y0) * log(x1/x) +\
+                                     log(y1) * log(x/x0))/log(x1/x0))
+                elif (y1 > 0) and ((x-x0) > 1.E-5):
+                    ddict[key][i] = exp((log(y1) * log(x/x0))/log(x1/x0))
             
 
         #partial cross sections
