@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 __revision__ = "$Revision: 2.02 $"
 #/*##########################################################################
-# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2013 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -28,6 +28,9 @@ __revision__ = "$Revision: 2.02 $"
 #############################################################################*/
 import sys, getopt
 import traceback
+if sys.platform == 'win32':
+    import ctypes
+    from ctypes.wintypes import MAX_PATH
 nativeFileDialogs = None
 DEBUG = 0
 if __name__ == '__main__':
@@ -78,12 +81,19 @@ else:
     QString = qt.safe_str
 
 QTVERSION = qt.qVersion()
+XRFMC_FLAG = False
+if QTVERSION > '4.0.0':
+    try:
+        from PyMca.XRFMC import XRFMCPyMca
+        XRFMC_FLAG = True
+    except:
+        pass
 
 from PyMca.PyMca_Icons import IconDict
 from PyMca.PyMca_help import HelpDict
 from PyMca import PyMcaDataDir
 import os
-__version__ = "4.6.0"
+__version__ = "4.7.1"
 if (QTVERSION < '4.0.0') and (sys.platform == 'darwin'):
     class SplashScreen(qt.QWidget):
         def __init__(self,parent=None,name="SplashScreen",
@@ -161,6 +171,7 @@ if __name__ == "__main__":
         else:
             splash = qt.QSplashScreen()
         splash.show()
+        splash.raise_()
         from PyMca import ChangeLog
         font = splash.font()
         font.setBold(1)
@@ -262,6 +273,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             self.__fit2Spec  = None
             self.__correlator  = None
             self.__imagingTool = None
+            self._xrfmcTool = None
             if QTVERSION < '4.0.0':
                 self.openMenu = qt.QPopupMenu()
                 self.openMenu.insertItem("PyMca Configuration",0)
@@ -383,8 +395,12 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                 else:
                     self.mcawindow.showMaximized()
             currentConfigDict = ConfigDict.ConfigDict()
-            defaultFileName = self.__getDefaultSettingsFile()
-            self.configDir  = os.path.dirname(defaultFileName)
+            try:
+                defaultFileName = self.__getDefaultSettingsFile()
+                self.configDir  = os.path.dirname(defaultFileName)
+            except:
+                if not ('fresh' in kw):
+                    raise
             if not ('fresh' in kw):
                 if os.path.exists(defaultFileName):
                     currentConfigDict.read(defaultFileName)
@@ -432,8 +448,10 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         else:
             legend = "%s" % self.mainTabWidget.tabText(index)
             for key in self.imageWindowDict.keys():
-                if key == legend:value = True
-                else: value = False
+                if key == legend:
+                    value = True
+                else:
+                    value = False
                 self.imageWindowDict[key].setPlotEnabled(value)
 
     def _is2DSelection(self, ddict):
@@ -650,7 +668,8 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             legend = ddict['legend']
             for key in list(self.imageWindowDict.keys()):
                 index = self.mainTabWidget.indexOf(self.imageWindowDict[key])
-                if key == legend:continue
+                if key == legend:
+                    continue
                 if index >= 0:
                     self.imageWindowDict[key].close()
                     self.imageWindowDict[key].setParent(None)
@@ -666,6 +685,10 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                     self.mainTabWidget.setCurrentPage(index)
                 else:
                     self.mainTabWidget.setCurrentWidget(self.imageWindowDict[legend])
+            if legend in self.imageWindowDict.keys():
+                # force an update
+                self.imageWindowDict[legend].setPlotEnabled(True)
+                self.imageWindowDict[legend].setPlotEnabled(False)
         elif self._is3DSelection(ddict):
             self.glWindow._replaceSelection(ddict)             
         else:
@@ -861,10 +884,15 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                 if 'SourceName' in dict[source]:
                     if type(dict[source]['SourceName']) != type([]):
                         dict[source]['SourceName'] = [dict[source]['SourceName'] * 1]
-                    for SourceName in dict[source]['SourceName']:
+                    for SourceName0 in dict[source]['SourceName']:
+                        if type(SourceName0) == type([]):
+                            SourceName = SourceName0[0]
+                        else:
+                            SourceName = SourceName0
                         if len(SourceName):
                             try:
-                                if not os.path.exists(SourceName): continue
+                                if not os.path.exists(SourceName):
+                                    continue
                                 self.sourceWidget.sourceSelector.openFile(SourceName, justloaded =1)
                                 continue
                                 #This event is not needed
@@ -880,10 +908,13 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                             except:
                                 msg = qt.QMessageBox(self)
                                 msg.setIcon(qt.QMessageBox.Critical)
-                                msg.setText("Error: %s\n opening file %s" % (sys.exc_info()[1],SourceName ))
+                                txt = "Error: %s\n opening file %s" % (sys.exc_info()[1],SourceName )
                                 if QTVERSION < '4.0.0':
+                                    msg.setText(txt)
                                     msg.exec_loop()
                                 else:
+                                    msg.setInformativeText(txt)
+                                    msg.setDetailedText(traceback.format_exc())
                                     msg.exec_()
 
                 if 'WidgetConfiguration' in dict[source]:
@@ -925,7 +956,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
 
     def __configureFit(self, d):
         if 'Configuration' in d:
-            self.mcawindow.advancedfit.mcafit.configure(d['Configuration'])
+            self.mcawindow.advancedfit.configure(d['Configuration'])
             if not self.mcawindow.advancedfit.isHidden():
                 self.mcawindow.advancedfit._updateTop()
         if 'ConfigDir' in d:
@@ -1183,7 +1214,8 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
 
     def _startupSelection(self, source, selection):
         self.sourceWidget.sourceSelector.openSource(source)
-        if selection is None:return
+        if selection is None:
+            return
         
         if len(selection) >= 8:
             if selection[0:8] == "MCA_DATA":
@@ -1249,6 +1281,9 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             if XIA_CORRECT:
                 self.menuTools.addAction("XIA Correct",
                                          self.__xiaCorrect)
+            if XRFMC_FLAG:
+                self.menuTools.addAction("XMI-MSIM PyMca",
+                                         self._xrfmcPyMca)                
         if DEBUG:
             print("Fit to Specfile missing")
             
@@ -1497,6 +1532,12 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
 
     def __xiaCorrect(self):
         XiaCorrect.mainGUI(qt.qApp)
+
+    def _xrfmcPyMca(self):
+        if self._xrfmcTool is None:
+            self._xrfmcTool = XRFMCPyMca.XRFMCPyMca()
+        self._xrfmcTool.show()
+        self._xrfmcTool.raise_()
     
     def onOpen(self):
         if QTVERSION < '4.0.0':
@@ -1601,15 +1642,22 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
     def __getDefaultSettingsFile(self):
         filename = "PyMca.ini"
         if sys.platform == 'win32':
-            home = os.getenv('USERPROFILE')
-            try:
-                l = len(home)
-                directory = os.path.join(home, "My Documents")
-            except:
-                home = '\\'
-                directory = '\\'
-            #print home
-            #print directory
+            # recipe based on: http://bugs.python.org/issue1763#msg62242
+            dll = ctypes.windll.shell32
+            buf = ctypes.create_unicode_buffer(MAX_PATH + 1)
+            if dll.SHGetSpecialFolderPathW(None, buf, 0x0005, False):
+                directory = buf.value
+            else:
+                # the above should have worked
+                home = os.getenv('USERPROFILE')
+                try:
+                    l = len(home)
+                    directory = os.path.join(home, "My Documents")
+                except:
+                    home = '\\'
+                    directory = '\\'
+                #print home
+                #print directory
             if os.path.isdir('%s' % directory):
                 directory = os.path.join(directory, "PyMca")
             else:
@@ -1683,8 +1731,10 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         source = SOURCESLIST[index]
         if self.sourceFrame.isHidden():
             self.sourceFrame.show()
-        if QTVERSION < '4.0.0':self.sourceFrame.raiseW()
-        else:self.sourceFrame.raise_()
+        if QTVERSION < '4.0.0':
+            self.sourceFrame.raiseW()
+        else:
+            self.sourceFrame.raise_()
         self.sourceBrowserTab.showPage(self.sourceWidget[source])
         qt.qApp.processEvents()
         self.sourceWidget[source].openFile()
@@ -1806,13 +1856,22 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
     def onPrint(self):
         if DEBUG:
             print("onPrint called")
-        if self.scanwindow.hasFocus():
-            self.scanwindow.graph.printps() 
-        else:
+        if not self.scanwindow.isHidden():
+            self.scanwindow.graph.printps()
+            return
+
+        if not self.__useTabWidget:
             self.mcawindow.show()
-            if QTVERSION < '4.0.0':self.mcawindow.raiseW()
-            else:self.mcawindow.raise_()
-            self.mcawindow.graph.printps()    
+            if QTVERSION < '4.0.0':
+                self.mcawindow.raiseW()
+            else:
+                self.mcawindow.raise_()
+        else:
+            if QTVERSION < '4.0.0':         
+                self.mainTabWidget.setCurrentPage(self.mainTab.indexOf(self.mcawindow))
+            else:
+                self.mainTabWidget.setCurrentWidget(self.mcawindow)
+        self.mcawindow.graph.printps()
 
     def __McaWindowSignal(self, ddict):
         if ddict['event'] == 'NewScanCurve':
@@ -1953,6 +2012,7 @@ if __name__ == '__main__':
     else:
         splash.finish(PyMcaMainWidgetInstance)
         PyMcaMainWidgetInstance.show()
+        PyMcaMainWidgetInstance.raise_()
         #try to interpret rest of command line arguments as data sources
         try:
             for source in args:
