@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2013 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -25,7 +25,7 @@
 # is a problem for you.
 #############################################################################*/
 """
-Demo example of generic access to data sources.
+Generic access to data sources.
 
 """
 import sys
@@ -33,19 +33,12 @@ import os
 from PyMca import PyMcaQt as qt
 QTVERSION = qt.qVersion()
 
-#import QSPSDataSource
-#import QSpecFileDataSource
-#import QEdfFileDataSource
 from PyMca import SpecFileDataSource
 from PyMca import EdfFileDataSource
 from PyMca import QEdfFileWidget
-if 0 and QTVERSION < '4.0.0':
-    from PyMca import MySpecFileSelector as QSpecFileWidget
-    QSpecFileWidget.QSpecFileWidget = QSpecFileWidget.SpecFileSelector
-else:
-    from PyMca import QSpecFileWidget
+from PyMca import QSpecFileWidget
 
-if (sys.platform == "win32") or (sys.platform == "darwin"):
+if sys.platform == "win32":
     source_types = { SpecFileDataSource.SOURCE_TYPE: SpecFileDataSource.SpecFileDataSource,
                      EdfFileDataSource.SOURCE_TYPE:  EdfFileDataSource.EdfFileDataSource}
 
@@ -53,7 +46,6 @@ if (sys.platform == "win32") or (sys.platform == "darwin"):
                        EdfFileDataSource.SOURCE_TYPE: QEdfFileWidget.QEdfFileWidget}
     sps = None 
 else:
-    #import SpsDataSource
     from PyMca import QSpsDataSource
     sps = QSpsDataSource.SpsDataSource.sps
     from PyMca import QSpsWidget
@@ -71,6 +63,7 @@ try:
     from PyMca import PyMcaNexusWidget
     import h5py
 except:
+    # HDF5 file format support is not mandatory
     NEXUS = False
 
 
@@ -129,13 +122,21 @@ def getSourceType(sourceName0):
         elif line[0:2] in ["II","MM"]:
             #this also accounts for TIFF
             mccd = True
-        elif sourceName.lower().endswith('.spe') and (line[0:1] != '$'):
+        elif sourceName.lower().endswith('.spe') and\
+             (line[0:1] not in ['$', '#']):
             #Roper images
             mccd = True
         else:
             mccd = False
         if (line.startswith("{")) or mccd:
             return EdfFileDataSource.SOURCE_TYPE
+        elif sourceName.lower().endswith('edf.gz') or\
+             sourceName.lower().endswith('ccd.gz') or\
+             sourceName.lower().endswith('raw.gz') or\
+             sourceName.lower().endswith('edf.bz2') or\
+             sourceName.lower().endswith('ccd.bz2') or\
+             sourceName.lower().endswith('raw.bz2'):
+            return EdfFileDataSource.SOURCE_TYPE
         else:
             if NEXUS:
                 ishdf5 = False
