@@ -29,12 +29,14 @@ __author__ = "V.A. Sole - ESRF Software Group"
 import sys
 import os
 import numpy
+import traceback
 
 from matplotlib import cm
 from matplotlib.font_manager import FontProperties
 from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
 from matplotlib.figure import Figure
 from matplotlib.colors import LinearSegmentedColormap, LogNorm, Normalize
+from matplotlib.ticker import MaxNLocator, AutoLocator
 
 from PyMca import PyMcaQt as qt
 from PyMca import PyMcaMatplotlibSave
@@ -173,7 +175,9 @@ class SaveImageSetup(qt.QWidget):
         except:
             msg = qt.QMessageBox(self)
             msg.setIcon(qt.QMessageBox.Critical)
-            msg.setText("Error updating image: %s" % sys.exc_info()[1])
+            msg.setText("Error updating image:")
+            msg.setInformativeText("%s" % sys.exc_info()[1])
+            msg.setDetailedText(traceback.format_exc())
             msg.setWindowTitle('Matplotlib Save Image')
             msg.exec_()
             
@@ -311,6 +315,8 @@ class RightWidget(qt.QWidget):
         self.gridLayout.setSpacing(2)
         self.labelList = ['X Axis',
                         'Y Axis',
+                        'N X Labels',
+                        'N Y Labels',
                         'Origin',
                         'Interpolation',
                         'Colormap',
@@ -320,6 +326,7 @@ class RightWidget(qt.QWidget):
                         'Contour Labels',
                         'Contour Label Format',
                         'Contour Levels',
+                        'Contour Line Width',
                         'Image Background',
                         'X Pixel Size',
                         'Y Pixel Size',
@@ -341,12 +348,19 @@ class RightWidget(qt.QWidget):
             label.setText(self.labelList[i])
             if self.labelList[i] in ['X Axis', 'Y Axis']:
                 options = ['Off', 'On']
+            if self.labelList[i] in ['N X Labels', 'N Y Labels']:
+                options = ['Auto', '1', '2', '3', '4', '5', '6', '7', '8', '9']
             elif self.labelList[i] in ['Colormap']:
                 options = ['Temperature','Grey', 'Yerg',\
                            'Red', 'Green', 'Blue',\
                            'Rainbow', 'Jet','Hot', 'Cool', 'Copper']
-                if hasattr(cm, 'spectral'):
-                    options.append('Spectral')
+                for candidate in ['spectral', 'Paired', 'Paired_r',
+                                  'PuBu', 'PuBu_r', 'RdBu', 'RdBu_r',
+                                  'gist_earth', 'gist_earth_r',
+                                  'Blues', 'Blues_r',
+                                  'YlGnBu', 'YlGnBu_r']:
+                    if hasattr(cm, candidate):
+                        options.append(candidate)
             elif self.labelList[i] in ['Lin/Log Colormap']:
                 options = ['Linear','Logarithmic']
             elif self.labelList[i] in ['Colorbar']:
@@ -365,7 +379,18 @@ class RightWidget(qt.QWidget):
                 options = ["10", "9", "8", "7", "6", "5", "4", "3", "2", "1"]
             elif self.labelList[i] in ['Image Background']:
                 options = ['Black', 'White', 'Grey']
-            if i <= self.labelList.index('Image Background'):
+                
+            if self.labelList[i] in ['Contour Levels']:
+                line = qt.QSpinBox(self)
+                line.setMinimum(1)
+                line.setMaximum(1000)
+                line.setValue(10)
+            elif self.labelList[i] in ['Contour Line Width']:
+                line = qt.QSpinBox(self)
+                line.setMinimum(1)
+                line.setMaximum(100)
+                line.setValue(10)
+            elif i <= self.labelList.index('Image Background'):
                 line = SimpleComboBox(self, options)
             else:
                 line = MyLineEdit(self)
@@ -422,11 +447,18 @@ class RightWidget(qt.QWidget):
         ddict = {}
         i = 0
         for label in self.keyList:
-            if i > self.labelList.index('Image Background'):
+            if i == self.labelList.index('Contour Levels'):
+                ddict[label] = self.comboBoxList[i].value()
+            elif i == self.labelList.index('Contour Line Width'):
+                ddict[label] = self.comboBoxList[i].value()
+            elif i > self.labelList.index('Image Background'):
                 text = qt.safe_str(self.comboBoxList[i].text())
                 if len(text):
-                    if label == 'Output dpi':
-                        ddict[label] = int(text)
+                    if label in ['Output dpi', "N X Labels", "N Y Labels"]:
+                        if ddict[label] in ['Auto', 'auto', '0', 0]:
+                            ddict['label'] = 0
+                        else:
+                            ddict[label] = int(text)
                     else:
                         ddict[label] = float(text)
                 else:
@@ -442,10 +474,20 @@ class RightWidget(qt.QWidget):
         for label in ddict.keys():
             if label.lower() in self.keyList:
                 i = self.keyList.index(label)
-                if i > self.labelList.index('Image Background'):
+                if i == self.labelList.index('Contour Levels'):
+                    self.comboBoxList[i].setValue(int(ddict[label]))
+                elif i == self.labelList.index('Contour Line Width'):
+                    self.comboBoxList[i].setValue(int(ddict[label]))
+                elif i > self.labelList.index('Image Background'):
                     if ddict[label] is not None:
-                        if label == 'Output dpi':
+                        if label in ['Output dpi']:
                             self.comboBoxList[i].setText("%d" % int(ddict[label]))
+                        elif label in ['N X Labels', 'N Y Labels']:
+                            if ddict[label] in ['Auto', 'auto', '0', 0]:
+                                self.comboBoxList[i].setText("Auto")
+                            else:
+                                self.comboBoxList[i].setText("%d" %\
+                                                             int(ddict[label]))
                         else:
                             self.comboBoxList[i].setText("%f" % ddict[label])
                 else:
@@ -458,7 +500,7 @@ class RightWidget(qt.QWidget):
                             pass
                     self.comboBoxList[i].setCurrentText(txt)
         return
-	
+
 class MyLineEdit(qt.QLineEdit):
     def sizeHint(self):
         return qt.QSize(0.6 * qt.QLineEdit.sizeHint(self).width(),
@@ -467,12 +509,14 @@ class MyLineEdit(qt.QLineEdit):
 
 class QPyMcaMatplotlibImage(FigureCanvas):
     def __init__(self, parent, imageData=None,
-		     dpi=100,
+                     dpi=100,
                      size=(5, 5),
                      xaxis='off',
                      yaxis='off',
                      xlabel='',
                      ylabel='',
+                     nxlabels=0,
+                     nylabels=0,
                      colorbar=None,
                      title='',
                      interpolation='nearest',
@@ -483,6 +527,7 @@ class QPyMcaMatplotlibImage(FigureCanvas):
                      contourlabels='on',
                      contourlabelformat='%.3f',                 
                      contourlevels=2,
+                     contourlinewidth=10,
                      extent=None,
                      xpixelsize=1.0,
                      ypixelsize=1.0,
@@ -509,6 +554,8 @@ class QPyMcaMatplotlibImage(FigureCanvas):
                      'title':title,
                      'xlabel':xlabel,
                      'ylabel':ylabel,
+                     'nxlabels':nxlabels,
+                     'nylabels':nylabels,
                      'colorbar':colorbar,
                      'colormap':colormap,
                      'linlogcolormap':linlogcolormap,
@@ -518,6 +565,7 @@ class QPyMcaMatplotlibImage(FigureCanvas):
                      'contourlabels':contourlabels,
                      'contourlabelformat':contourlabelformat,
                      'contourlevels':contourlevels,
+                     'contourlinewidth':contourlinewidth,
                      'extent':extent,
                      'imagebackground':'black',
                      'xorigin':xorigin,
@@ -595,16 +643,26 @@ class QPyMcaMatplotlibImage(FigureCanvas):
            (self.pixmapImage is None):
             return
 
-	# The axes
+        # The axes
         self.axes = self.figure.add_axes([.15, .15, .75, .8])
         if self.config['xaxis'] == 'off':
             self.axes.xaxis.set_visible(False)
         else:
             self.axes.xaxis.set_visible(True)
+            nLabels = self.config['nxlabels']
+            if nLabels not in ['Auto', 'auto', '0', 0]:
+                self.axes.xaxis.set_major_locator(MaxNLocator(nLabels))
+            else:
+                self.axes.xaxis.set_major_locator(AutoLocator())
         if self.config['yaxis'] == 'off':
             self.axes.yaxis.set_visible(False)
         else:
             self.axes.yaxis.set_visible(True)
+            nLabels = self.config['nylabels']
+            if nLabels not in ['Auto', 'auto', '0', 0]:
+                self.axes.yaxis.set_major_locator(MaxNLocator(nLabels))
+            else:
+                self.axes.yaxis.set_major_locator(AutoLocator())
 
         if self.pixmapImage is not None:
             self._updatePixmapFigure()
@@ -643,6 +701,33 @@ class QPyMcaMatplotlibImage(FigureCanvas):
             cmap = self.__blueCmap
         elif self.config['colormap']=='temperature':
             cmap = self.__temperatureCmap
+        elif self.config['colormap'] == 'paired':
+            cmap = cm.Paired
+        elif self.config['colormap'] == 'paired_r':
+            cmap = cm.Paired_r
+        elif self.config['colormap'] == 'pubu':
+            cmap = cm.PuBu
+        elif self.config['colormap'] == 'pubu_r':
+            cmap = cm.PuBu_r
+        elif self.config['colormap'] == 'rdbu':
+            cmap = cm.RdBu
+        elif self.config['colormap'] == 'rdbu_r':
+            cmap = cm.RdBu_r
+        elif self.config['colormap'] == 'gist_earth':
+            cmap = cm.gist_earth
+        elif self.config['colormap'] == 'gist_earth_r':
+            cmap = cm.gist_earth_r
+        elif self.config['colormap'] == 'blues':
+            cmap = cm.Blues
+        elif self.config['colormap'] == 'blues_r':
+            cmap = cm.Blues_r
+        elif self.config['colormap'] == 'ylgnbu':
+            cmap = cm.YlGnBu
+        elif self.config['colormap'] == 'ylgnbu_r':
+            cmap = cm.YlGnBu_r
+        else:
+            print("Unsupported colormap %s" % self.config['colormap'])
+
 
         if self.config['extent'] is None:
             h, w = self.imageData.shape
@@ -704,7 +789,8 @@ class QPyMcaMatplotlibImage(FigureCanvas):
             dataMax = imageData.max()
             ncontours = int(self.config['contourlevels'])
             levels = (numpy.arange(ncontours)) *\
-                     (dataMax - dataMin)/float(ncontours)	    
+                     (dataMax - dataMin)/float(ncontours)
+            contourlinewidth = int(self.config['contourlinewidth'])/10.
             if self.config['contour'] == 'filled':
                 self._contour = self.axes.contourf(imageData, levels,
                      origin=origin,
@@ -714,7 +800,7 @@ class QPyMcaMatplotlibImage(FigureCanvas):
                 self._contour = self.axes.contour(imageData, levels,
                      origin=origin,
                      cmap=ccmap,
-                     linewidths=2,
+                     linewidths=contourlinewidth,
                      extent=extent)
             if self.config['contourlabels'] != 'off':
                 self.axes.clabel(self._contour, fontsize=9,
