@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #/*##########################################################################
-# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2013 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -598,7 +598,7 @@ class QStackWidget(StackBase.StackBase,
             idx = 0
         else:
             actionList = []
-            methods.sort()
+            #methods.sort()
             menu = qt.QMenu(self)
             for method in methods:
                 text = QString(method)
@@ -621,7 +621,7 @@ class QStackWidget(StackBase.StackBase,
                 if a.text() == action[0]:
                     idx = actionList.index(action)
         try:
-            self.pluginInstanceDict[key].applyMethod(methods[idx])    
+            self.pluginInstanceDict[key].applyMethod(methods[idx])
         except:
             msg = qt.QMessageBox(self)
             msg.setIcon(qt.QMessageBox.Critical)
@@ -752,8 +752,16 @@ class QStackWidget(StackBase.StackBase,
     def calculateROIImages(self, index1, index2, imiddle=None, energy=None):
         #overwrite base method to update the default energy with the one
         # currently used in the graph
-        # TODO: Handle the case of not having an active curve!!!
-        x, y, legend, info = self.mcaWidget.getActiveCurve()
+        activeCurve = self.mcaWidget.getActiveCurve()
+        if activeCurve is None:
+            msg = qt.QMessageBox(self)
+            msg.setIcon(qt.QMessageBox.Information)
+            msg.setWindowTitle("No active curve selected")
+            text  = "Please select the MCA active curve."
+            msg.setText(text)
+            msg.exec_()
+            return
+        x, y, legend, info = activeCurve
         return StackBase.StackBase.calculateROIImages(self,
                                                       index1,
                                                       index2,
