@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2013 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -302,6 +302,18 @@ class RGBCorrelatorGraph(qt.QWidget):
     def infoText(self):
         return self.infoWidget.label.text()
 
+    def setXLabel(self, label="Column"):
+        return self.graph.x1Label(label)
+
+    def setYLabel(self, label="Row"):
+        return self.graph.ylabel(label)
+
+    def getXLabel(self):
+        return self.graph.x1Label()
+
+    def getYLabel(self):
+        return self.graph.y1Label()
+
     def hideImageIcons(self):
         if self.imageToolButton is None:return
         self.imageToolButton.hide()
@@ -386,6 +398,17 @@ class RGBCorrelatorGraph(qt.QWidget):
         self._pickerSelectionWidthLabel.show()
         self._pickerSelectionWidthValue.show()
 
+    def getPickerSelectionMode(self):
+        if not len(self._pickerSelectionButtons):
+            return None
+        if self.hLineProfileButton.isChecked():
+            return "HORIZONTAL"
+        if self.vLineProfileButton.isChecked():
+            return "VERTICAL"
+        if self.lineProfileButton.isChecked():
+            return "LINE"
+        return None
+
     def _setPickerSelectionMode(self, mode=None):
         if mode is None:
             self.graph.setPickerSelectionModeOff()
