@@ -7,8 +7,30 @@
 # and are very grateful to Uwe for making his code available to the
 # community.
 #
+#    Copyright (C) 2008 Uwe Schmitt
 #
-
+# Permission is hereby granted, free of charge, to any person
+# obtaining a copy of this software and associated documentation
+# files (the "Software"), to deal in the Software without
+# restriction, including without limitation the rights to use,
+# copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the
+# Software is furnished to do so, subject to the following
+# conditions:
+#
+# The above copyright notice and this permission notice shall be
+# included in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+# OTHER DEALINGS IN THE SOFTWARE.
+#
+__license__ = "MIT"
 import numpy
 from numpy.linalg import solve
 
@@ -76,8 +98,6 @@ def getSavitzkyGolay(spectrum, npoints=3, degree=1, order=0):
     return result
 
 def replaceStackWithSavitzkyGolay(stack, npoints=3, degree=1, order=0):
-    #Warning: Not checked if the last dimension is the one containing the
-    #spectra as it is assuming!!!!
     coeff = calc_coeff(npoints, degree, order)
     N = numpy.size(coeff-1)/2
     convolve = numpy.convolve
@@ -103,7 +123,7 @@ def replaceStackWithSavitzkyGolay(stack, npoints=3, degree=1, order=0):
     elif mcaIndex == 0:
         data.shape = oldShape[0], -1
         for i in range(data.shape[-1]):
-            data[N:-N, i] = convolve(data[i,:],coeff, mode='valid')
+            data[N:-N, i] = convolve(data[:, i],coeff, mode='valid')
             if order > 0:
                 data[:N, i] = data[N, i]
                 data[-N:, i] = data[-(N+1), i]            
