@@ -1,51 +1,51 @@
 #/*##########################################################################
-# Copyright (C) 2004-2009 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
 #
-# This file is part of the PyMCA X-ray Fluorescence Toolkit developed at
-# the ESRF by the Beamline Instrumentation Software Support (BLISS) group.
+# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
+# the ESRF by the Software group.
 #
-# This toolkit is free software; you can redistribute it and/or modify it 
+# This toolkit is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the Free
-# Software Foundation; either version 2 of the License, or (at your option) 
+# Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
 #
-# PyMCA is distributed in the hope that it will be useful, but WITHOUT ANY
+# PyMca is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 # FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 # details.
 #
 # You should have received a copy of the GNU General Public License along with
-# PyMCA; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
-# Suite 330, Boston, MA 02111-1307, USA.
+# PyMca; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
-# PyMCA follows the dual licensing model of Trolltech's Qt and Riverbank's PyQt
-# and cannot be used as a free plugin for a non-free program. 
+# PyMca follows the dual licensing model of Riverbank's PyQt and cannot be
+# used as a free plugin for a non-free program.
 #
-# Please contact the ESRF industrial unit (industry@esrf.fr) if this license 
+# Please contact the ESRF industrial unit (industry@esrf.fr) if this license
 # is a problem for you.
 #############################################################################*/
-__author__ = "V.A. Sole - ESRF BLISS Group"
-import PyMcaQt as qt
-from PyMca_Icons import IconDict
-import MaskImageWidget
-#RGBCorrelatorGraph
-import ScanWindow
+__author__ = "V.A. Sole - ESRF Software Group"
 import sys
-import SNIPModule
+from PyMca import PyMcaQt as qt
+from PyMca.PyMca_Icons import IconDict
+from PyMca import MaskImageWidget
+from PyMca import ScanWindow
+from PyMca import SNIPModule
 OBJECT3D = False
 try:
-    from Object3D import Object3DScene
+    from PyMca.Object3D import Object3DScene
     OBJECT3D = True
 except:
-    OBJECT3D = False
+    try:
+        # Frozen version handling
+        from Object3D import Object3DScene
+        OBJECT3D = True
+    except:
+        OBJECT3D = False
+
 
-class HorizontalSpacer(qt.QWidget):
-    def __init__(self, *args):
-        qt.QWidget.__init__(self, *args)
-        self.setSizePolicy(qt.QSizePolicy(qt.QSizePolicy.Expanding,
-                                          qt.QSizePolicy.Fixed))
 class SNIP1DParametersWidget(qt.QWidget):
-    def __init__(self, parent = None, length=2000):
+    def __init__(self, parent = None, length=2000, smooth=False):
         qt.QWidget.__init__(self, parent)
         self.mainLayout = qt.QGridLayout(self)
         self.mainLayout.setMargin(0)
@@ -56,14 +56,19 @@ class SNIP1DParametersWidget(qt.QWidget):
                                'roi_max':length,
                                'width':min(30, length/10),
                                'smoothing':1}
-        for text in ["SNIP background width (2 to 3 times fwhm) :",
+        textLabels = ["SNIP background width (2 to 3 times fwhm) :",
                      "Minimum channel considered:",
                      "Maximum channel considered:",
-                     "Pleliminar smoothing level:"]:
+                     "Preliminar smoothing level:"]
+        if smooth:
+            textLabels[0] = "SNIP width :"
+            self.parametersDict['width'] = 3
+            
+        for text in textLabels:
             label = qt.QLabel(self)
             label.setText(text)
             self.mainLayout.addWidget(label, i, 0)        
-            #self.mainLayout.addWidget(HorizontalSpacer(self), i, 1)
+            #self.mainLayout.addWidget(qt.HorizontalSpacer(self), i, 1)
             i +=1 
 
         i = 0
@@ -91,6 +96,18 @@ class SNIP1DParametersWidget(qt.QWidget):
     def getParameters(self):
         return self.parametersDict
 
+    def setParameters(self, ddict=None):
+        if ddict is None:
+            return        
+        actualKeys = self.widgetDict.keys()
+        for key in ddict.keys():
+            if key in actualKeys:
+                w = self.widgetDict[key] 
+                #w.setMaximum(max(ddict[key], w.value()))
+                #w.setMinimum(min(ddict[key], w.value()))
+                w.setValue(ddict[key])
+        self._updateParameters("dummy")
+
 class SNIP2DParametersWidget(qt.QWidget):
     def __init__(self, parent = None, shape=(4000,4000)):
         qt.QWidget.__init__(self, parent)
@@ -110,7 +127,7 @@ class SNIP2DParametersWidget(qt.QWidget):
             label = qt.QLabel(self)
             label.setText(text)
             self.mainLayout.addWidget(label, i, 0)        
-            #self.mainLayout.addWidget(HorizontalSpacer(self), i, 1)
+            #self.mainLayout.addWidget(qt.HorizontalSpacer(self), i, 1)
             i +=1 
 
         i = 0
@@ -183,8 +200,11 @@ class SNIP2DParametersWidget(qt.QWidget):
     def getParameters(self):
         return self.parametersDict
 
+    def setParameters(self):
+        raise NotImplemented("Set parameters not implemented for SNIP 2D")
+
 class SNIPWindow(qt.QWidget):
-    def __init__(self, parent, data, image=None):
+    def __init__(self, parent, data, image=None, x=None, smooth=False):
         qt.QWidget.__init__(self, parent)
         self.setWindowTitle("SNIP Configuration Window")
         self.mainLayout = qt.QVBoxLayout(self)
@@ -201,9 +221,16 @@ class SNIPWindow(qt.QWidget):
                 spectrum = data
         elif not image:
             spectrum = data
+        self.__smooth = smooth
         self.__image = image
         if self.__image:
             self.spectrum = None
+        else:
+            if x is None:
+                self.xValues = range(len(spectrum))
+            else:
+                self.xValues = x
+        if self.__image:
             self.image = data
             self.graphWidget = MaskImageWidget.MaskImageWidget(self,
                                                             colormap=True,
@@ -219,15 +246,18 @@ class SNIPWindow(qt.QWidget):
         else:
             self.image = None
             self.spectrum = spectrum
-            self.parametersWidget = SNIP1DParametersWidget(self, length=len(spectrum))
+            self.parametersWidget = SNIP1DParametersWidget(self,
+                                                           length=len(spectrum),
+                                                           smooth=smooth)
             self.graph = ScanWindow.ScanWindow(self)
-            self.graph.newCurve(range(len(spectrum)),
+            self.graph.newCurve(self.xValues,
                             spectrum, "Spectrum", replace=True)
             self.mainLayout.addWidget(self.parametersWidget)
             self.mainLayout.addWidget(self.graph)
         self.xMarkers = []
         self.yMarkers = []
         self.getParameters = self.parametersWidget.getParameters
+        self.setParameters = self.parametersWidget.setParameters
         self.connect(self.parametersWidget,
                      qt.SIGNAL('SNIPParametersSignal'),
                      self.updateGraph)
@@ -277,23 +307,28 @@ class SNIPWindow(qt.QWidget):
                                                    roi_min=roi_min,
                                                    roi_max=roi_max,
                                                    smoothing=smoothing)
-            self.graph.newCurve(range(len(self.spectrum)),
-                            self.background, "Background", replace=False)
+            if self.__smooth:
+                legend0 = "Smoothed Spectrum"
+            else:
+                legend0 = "Background"
+            self.graph.newCurve(self.xValues,
+                            self.background, legend0, replace=False)
         
             #Force information update
             legend = self.graph.getActiveCurve(just_legend=True)
-            if legend.startswith('Background'):
+            if legend.startswith(legend0[0:5]):
                 self.graph.setActiveCurve(legend)
 
 
 class SNIPDialog(qt.QDialog):
-    def __init__(self, parent, data):
+    def __init__(self, parent, data, x=None, smooth=False):
         qt.QDialog.__init__(self, parent)
         self.setWindowTitle("SNIP Configuration Dialog")
         self.mainLayout = qt.QVBoxLayout(self)
         self.mainLayout.setMargin(10)
         self.mainLayout.setSpacing(2)
         self.__image = False
+        self.__smooth = smooth
         if len(data.shape) == 2:
             if 1 not in data.shape:
                 image = data
@@ -303,9 +338,11 @@ class SNIPDialog(qt.QDialog):
         else:
             spectrum = data
         if self.__image:
-            self.parametersWidget = SNIPWindow(self, image, image=True)
+            self.parametersWidget = SNIPWindow(self, image, image=True, x=x)
         else:
-            self.parametersWidget = SNIPWindow(self, spectrum, image=False)
+            self.parametersWidget = SNIPWindow(self, spectrum, image=False,
+                                               x=x, smooth=smooth)
+        self.graph = self.parametersWidget.graph
         self.mainLayout.addWidget(self.parametersWidget)
         hbox = qt.QWidget(self)
         hboxLayout = qt.QHBoxLayout(hbox)
@@ -318,7 +355,7 @@ class SNIPDialog(qt.QDialog):
         self.dismissButton.setText("Cancel")
         self.dismissButton.setAutoDefault(False)
         hboxLayout.addWidget(self.okButton)
-        hboxLayout.addWidget(HorizontalSpacer(hbox))
+        hboxLayout.addWidget(qt.HorizontalSpacer(hbox))
         hboxLayout.addWidget(self.dismissButton)
         self.mainLayout.addWidget(hbox)
         self.connect(self.dismissButton, qt.SIGNAL("clicked()"), self.reject)
@@ -328,13 +365,26 @@ class SNIPDialog(qt.QDialog):
         parametersDict = self.parametersWidget.getParameters()
         if self.__image:
             parametersDict['function'] = SNIPModule.subtractSnip2DBackgroundFromStack
+        elif self.__smooth:
+            parametersDict['function'] = SNIPModule.replaceStackWithSnip1DBackground            
         else:
             parametersDict['function'] = SNIPModule.subtractSnip1DBackgroundFromStack
         parametersDict['arguments'] = [parametersDict['width'],
                                        parametersDict['roi_min'],
                                        parametersDict['roi_max'],
                                        parametersDict['smoothing']]
-        return parametersDict                                       
+        return parametersDict
+
+    def setParameters(self, ddict0):
+        if 'arguments' in ddict0:
+            ddict = {}
+            ddict['width'] = ddict0['arguments'][0]
+            ddict['roi_min'] = ddict0['arguments'][1]
+            ddict['roi_max'] = ddict0['arguments'][2]
+            ddict['smoothing'] = ddict0['arguments'][3]
+            self.parametersWidget.setParameters(ddict)
+        else:
+            self.parametersWidget.setParameters(ddict0)
                  
 if __name__ == "__main__":
     import numpy
@@ -344,7 +394,7 @@ if __name__ == "__main__":
         y=numpy.arange(1000.)
         w = SNIPDialog(None, y+numpy.sqrt(y)* noise)
     elif len(sys.argv) > 1:
-        import EdfFile
+        from PyMca import EdfFile
         edf = EdfFile.EdfFile(sys.argv[1])
         data = edf.GetData(0)
         w = SNIPDialog(None, data)    
@@ -357,4 +407,4 @@ if __name__ == "__main__":
     w.show()
     ret=w.exec_()
     if ret:
-        print w.getParameters()
+        print(w.getParameters())
