@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2013 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -392,7 +392,8 @@ class ScanWindow(qt.QWidget, Plot1DBase.Plot1DBase):
             idx = 0
         else:
             actionList = []
-            methods.sort()
+            # allow the plugin designer to specify the order
+            #methods.sort()
             menu = qt.QMenu(self)
             for method in methods:
                 text = QString(method)
@@ -1100,7 +1101,6 @@ class ScanWindow(qt.QWidget, Plot1DBase.Plot1DBase):
         return tmpstr
         
     def __QSimpleOperation(self, operation):
-        return self.__simpleOperation(operation)
         try:
             self.__simpleOperation(operation)
         except:
@@ -1416,7 +1416,9 @@ class ScanWindow(qt.QWidget, Plot1DBase.Plot1DBase):
             sel['scanselection'] = True
             sel['selection'] = copy.deepcopy(dataObject.info['selection'])
             sel['selectiontype'] = "1D"
-            if operation != 'fit':
+            if operation == 'average':
+                self._replaceSelection([sel])
+            elif operation != 'fit':
                 self._addSelection([sel])
             else:
                 self.__fitDataObject = newDataObject
@@ -1719,6 +1721,12 @@ class ScanWindow(qt.QWidget, Plot1DBase.Plot1DBase):
         else:
             return self.graph.getY2AxisLimits()
 
+    def setGraphXLimits(self, xmin, xmax, replot=False):
+        return self.graph.setX1AxisLimits(xmin, xmax, replot=replot)
+
+    def setGraphYLimits(self, ymin, ymax, replot=False):
+        return self.graph.setY1AxisLimits(ymin, ymax, replot=replot)
+
     def setActiveCurve(self, legend):
         self.graph.setActiveCurve(legend)
         ddict = {}
@@ -1825,7 +1833,7 @@ class ScanWindow(qt.QWidget, Plot1DBase.Plot1DBase):
             comment = info['scan'].get('scan', None)+"\n"
             h, k, l = info['scan'].get('hkl')
             if h != "----":
-                comment += "H = %s  K = %s  L = %s\n" % (k, k, l)
+                comment += "H = %s  K = %s  L = %s\n" % (h, k, l)
             peak   = info['graph']['peak']
             peakAt = info['graph']['peakat']
             fwhm   = info['graph']['fwhm']
