@@ -1,45 +1,43 @@
 #/*##########################################################################
-# Copyright (C) 2004-2010 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
 #
-# This file is part of the PyMCA X-ray Fluorescence Toolkit developed at
-# the ESRF by the Beamline Instrumentation Software Support (BLISS) group.
+# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
+# the ESRF by the Software group.
 #
-# This toolkit is free software; you can redistribute it and/or modify it 
+# This toolkit is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the Free
-# Software Foundation; either version 2 of the License, or (at your option) 
+# Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
 #
-# PyMCA is distributed in the hope that it will be useful, but WITHOUT ANY
+# PyMca is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 # FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 # details.
 #
 # You should have received a copy of the GNU General Public License along with
-# PyMCA; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
-# Suite 330, Boston, MA 02111-1307, USA.
+# PyMca; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
-# PyMCA follows the dual licensing model of Trolltech's Qt and Riverbank's PyQt
-# and cannot be used as a free plugin for a non-free program. 
+# PyMca follows the dual licensing model of Riverbank's PyQt and cannot be
+# used as a free plugin for a non-free program.
 #
-# Please contact the ESRF industrial unit (industry@esrf.fr) if this license 
+# Please contact the ESRF industrial unit (industry@esrf.fr) if this license
 # is a problem for you.
 #############################################################################*/
 import sys
-import SimpleFitControlWidget
-qt = SimpleFitControlWidget.qt
-HorizontalSpacer = SimpleFitControlWidget.HorizontalSpacer
-VerticalSpacer = SimpleFitControlWidget.VerticalSpacer
-import ConfigDict
-import PyMca_Icons as Icons
 import os.path
-import PyMcaDirs
+import numpy
+from PyMca import SimpleFitControlWidget
+
+from PyMca import PyMcaQt as qt
+from PyMca import ConfigDict
+from PyMca import PyMca_Icons as Icons
+from PyMca import PyMcaDirs
 
 #strip background handling
-import StripBackgroundWidget
-SCANWINDOW = StripBackgroundWidget.SCANWINDOW
-import numpy
+from PyMca import StripBackgroundWidget
+from PyMca import Parameters
 
-import Parameters
 DEBUG = 0
 
 class DummyWidget(qt.QWidget):
@@ -51,9 +49,9 @@ class DummyWidget(qt.QWidget):
         self.label = qt.QLabel(self)
         self.label.setAlignment(qt.Qt.AlignHCenter)
         self.label.setText(text)
-        self.mainLayout.addWidget(VerticalSpacer(self))
+        self.mainLayout.addWidget(qt.VerticalSpacer(self))
         self.mainLayout.addWidget(self.label)
-        self.mainLayout.addWidget(VerticalSpacer(self))
+        self.mainLayout.addWidget(qt.VerticalSpacer(self))
         self._configuration = {}
 
     def setConfiguration(self, ddict):
@@ -87,6 +85,8 @@ class DefaultParametersWidget(qt.QWidget):
         if ddict['configuration']['estimation'] is None:
             #initialize with the default parameters
             parameters = ddict['parameters']
+            if type(parameters) == type(""):
+                parameters = [parameters]
             xmin = self.simpleFitInstance._x.min()
             xmax = self.simpleFitInstance._x.max()
             if self.background:
@@ -109,6 +109,8 @@ class DefaultParametersWidget(qt.QWidget):
                 paramlist.append(paramdict)
         else:
             parameters = ddict['configuration']['estimation']['parameters']
+            if type(parameters) == type(""):
+                parameters = [parameters]
             paramlist = []
             for parameter in parameters:
                 paramdict = ddict['configuration']['estimation'][parameter]              
@@ -175,7 +177,7 @@ class SimpleFitConfigurationGUI(qt.QDialog):
             self.mainLayout.addWidget(self.tabWidget)
             self._stripDialog = None
         self.buildAndConnectActions()
-        self.mainLayout.addWidget(VerticalSpacer(self))
+        self.mainLayout.addWidget(qt.VerticalSpacer(self))
         self._fitFunctionWidgets = {}
         self._backgroundWidgets = {}
         self.setSimpleFitInstance(fit)
@@ -184,7 +186,7 @@ class SimpleFitConfigurationGUI(qt.QDialog):
 
     def _fitControlSlot(self, ddict):
         if DEBUG:
-            print "FitControlSignal", ddict
+            print("FitControlSignal", ddict)
         event = ddict['event']
         if event == "stripSetupCalled":
             if self._stripDialog is None:
@@ -327,9 +329,33 @@ class SimpleFitConfigurationGUI(qt.QDialog):
             self.setConfiguration(self.simpleFitInstance.getConfiguration())
 
     def setConfiguration(self, ddict):
-        if ddict.has_key('fit'):
+        currentConfig = self.simpleFitInstance.getConfiguration()
+        currentFiles  = []
+        for functionName in currentConfig['functions'].keys():
+            fname = currentConfig['functions'][functionName]['file']
+            if fname not in currentFiles:
+                currentFiles.append(fname)
+
+        if 'functions' in ddict:
+            #make sure new modules are imported
+            for functionName in ddict['functions'].keys():
+                fileName = ddict['functions'][functionName]['file']
+                if fileName not in currentFiles:
+                    try:
+                        if DEBUG:
+                            print("Adding file %s" % fileName)
+                        self.simpleFitInstance.importFunctions(fileName)
+                        currentFiles.append(fileName)
+                    except:
+                        if "library.zip" in fileName:
+                            if DEBUG:
+                                print("Assuming PyMca supplied fit function")
+                            continue
+                        print("Cannot import file %s" % fileName)
+                        print(sys.exc_info()[1])
+            
+        if 'fit' in ddict:
             self.fitControlWidget.setConfiguration(ddict['fit'])
-            currentConfig = self.simpleFitInstance.getConfiguration()
             fitFunction = ddict['fit']['fit_function']
             background = ddict['fit']['background_function']
             if fitFunction not in self._fitFunctionWidgets.keys():
@@ -343,17 +369,19 @@ class SimpleFitConfigurationGUI(qt.QDialog):
             fname = ddict['fit']['fit_function']
             widget = self._fitFunctionWidgets[fname]
             if fname not in [None, "None", "NONE"]:
-                if ddict['functions'].has_key(fname):
+                if fname in ddict['functions']:
                     #if currentConfig['functions'][fname]['widget'] is not None:
                         widget.setConfiguration(ddict['functions'][fname])
+                        self.fitFunctionWidgetStack.mainLayout.setCurrentWidget(widget)
             
             #background function
             fname = ddict['fit']['background_function']
             widget = self._backgroundWidgets[fname]
             if fname not in [None, "None", "NONE"]:
-                if ddict['functions'].has_key(fname):
+                if fname in ddict['functions']:
                     #if currentConfig['functions'][fname]['widget'] is not None:
                         widget.setConfiguration(ddict['functions'][fname])
+                        self.backgroundWidgetStack.mainLayout.setCurrentWidget(widget)
 
     def getConfiguration(self):
         oldConfiguration = self.simpleFitInstance.getConfiguration()
@@ -367,10 +395,12 @@ class SimpleFitConfigurationGUI(qt.QDialog):
         widget = self._fitFunctionWidgets[fname]
         if fname not in [None, "None", "NONE"]:
             ddict['functions'][fname]={}
+            ddict['functions'][fname]['file'] = \
+                oldConfiguration['functions'][fname]['file']                
             ddict['functions'][fname]['configuration'] =\
                 oldConfiguration['functions'][fname]['configuration']
             newConfig = widget.getConfiguration()
-            if newConfig.has_key('configuration'):
+            if 'configuration' in newConfig:
                 ddict['functions'][fname]['configuration'].update(\
                                         newConfig['configuration'])
             else:
@@ -381,10 +411,12 @@ class SimpleFitConfigurationGUI(qt.QDialog):
         widget = self._backgroundWidgets[fname]
         if fname not in [None, "None", "NONE"]:
             ddict['functions'][fname]={}
+            ddict['functions'][fname]['file'] = \
+                oldConfiguration['functions'][fname]['file']                
             ddict['functions'][fname]['configuration'] =\
                 oldConfiguration['functions'][fname]['configuration']
             newConfig = widget.getConfiguration()
-            if newConfig.has_key('configuration'):
+            if 'configuration' in newConfig:
                 ddict['functions'][fname]['configuration'].update(\
                                         newConfig['configuration'])
             else:
@@ -410,7 +442,7 @@ class SimpleFitConfigurationGUI(qt.QDialog):
                         "Choose fit configuration file",
                         initdir,
                         "Fit configuration files (*.cfg)\nAll Files (*)")
-            filename = str(filename)
+            filename = qt.safe_str(filename)
             if len(filename):
                 self.loadConfiguration(filename)
                 self.initDir = os.path.dirname(filename)
@@ -427,7 +459,7 @@ class SimpleFitConfigurationGUI(qt.QDialog):
                         "Choose fit configuration file",
                         initdir,
                         "Fit configuration files (*.cfg)\nAll Files (*)")
-            filename = str(filename)
+            filename = qt.safe_str(filename)
             if len(filename):
                 self.loadConfiguration(filename)
                 self.initDir = os.path.dirname(filename)
@@ -448,7 +480,7 @@ class SimpleFitConfigurationGUI(qt.QDialog):
                         "Enter output fit configuration file",
                         initdir,
                         "Fit configuration files (*.cfg)\nAll Files (*)")
-            filename = str(filename)
+            filename = qt.safe_str(filename)
             if len(filename):
                 if len(filename) < 4:
                     filename = filename+".cfg"
@@ -469,7 +501,7 @@ class SimpleFitConfigurationGUI(qt.QDialog):
                         "Enter output fit configuration file",
                         initdir,
                         "Fit configuration files (*.cfg)\nAll Files (*)")
-            filename = str(filename)
+            filename = qt.safe_str(filename)
             if len(filename):
                 if len(filename) < 4:
                     filename = filename+".cfg"
@@ -481,24 +513,21 @@ class SimpleFitConfigurationGUI(qt.QDialog):
 
     def loadConfiguration(self, filename):
         cfg= ConfigDict.ConfigDict()
-        if DEBUG:
+        try:
             cfg.read(filename)
             self.initDir = os.path.dirname(filename)
             self.setConfiguration(cfg)
-        else:
-            try:
-                cfg.read(filename)
-                self.initDir = os.path.dirname(filename)
-                self.setConfiguration(cfg)
-            except:
-                qt.QMessageBox.critical(self, "Load Parameters",
-                    "ERROR while loading parameters from\n%s"%filename, 
-                    qt.QMessageBox.Ok,
-                    qt.QMessageBox.NoButton,
-                    qt.QMessageBox.NoButton)
+        except:
+            if DEBUG:
+                raise
+            qt.QMessageBox.critical(self, "Load Parameters",
+                "ERROR while loading parameters from\n%s"%filename, 
+                qt.QMessageBox.Ok,
+                qt.QMessageBox.NoButton,
+                qt.QMessageBox.NoButton)
         
     def saveConfiguration(self, filename):
-        cfg= ConfigDict.ConfigDict(self.getConfiguration())
+        cfg = ConfigDict.ConfigDict(self.getConfiguration())
         if DEBUG:
             cfg.write(filename)
             self.initDir = os.path.dirname(filename)
@@ -523,7 +552,7 @@ def test():
     ddict['fit']['xmax'] = 1024
     wid.setConfiguration(ddict)
     wid.exec_()
-    print wid.getConfiguration()
+    print(wid.getConfiguration())
     sys.exit()
 
 if __name__=="__main__":
