@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2013 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -366,9 +366,10 @@ class SpecFileDataSource:
                       with a 3D array (pts_mot1*pts_mot2*mca)
         """
         key_type= self.__getKeyType(key)
-        if key_type=="scan": scan_key= key
-        elif key_type=="mca": (scan_key, mca_no)=self.__getMcaPars(key)
-
+        if key_type=="scan":
+            scan_key= key
+        elif key_type=="mca":
+            (scan_key, mca_no)=self.__getMcaPars(key)
         if self.__source_info_cached is None:
             sourceinfo = self.getSourceInfo()
             sourcekeys = sourceinfo['KeyList']
@@ -416,7 +417,8 @@ class SpecFileDataSource:
                         if label not in output.info['LabelNames']:
                             raise ValueError("Label %s not in scan labels" % label)
                         index = output.info['LabelNames'].index(label)
-                        if output.x is None: output.x = []
+                        if output.x is None:
+                            output.x = []
                         output.x.append(output.data[:, index])
                         indexlist.append(index)
                     output.info['selection']['x'] = indexlist
@@ -445,7 +447,8 @@ class SpecFileDataSource:
                         if label not in output.info['LabelNames']:
                             raise ValueError("Label %s not in scan labels" % label)
                         index = output.info['LabelNames'].index(label)
-                        if output.m is None: output.m = []
+                        if output.m is None:
+                            output.m = []
                         output.m.append(output.data[:, index])
                         indexlist.append(index)
                     output.info['selection']['m'] = indexlist
@@ -479,7 +482,7 @@ class SpecFileDataSource:
                     SPECFILE = False
                 for i in range(npoints):
                     if SPECFILE:
-                        wmca_no= (mca_no-1)*output.info['NbMcaDet'] + int(i+1)
+                        wmca_no= mca_no + output.info['NbMcaDet'] * i
                         mcaData= scan_obj.mca(wmca_no)
                     else:
                         mca_key = '%s.%d' % (scan_key, mca_no)
