@@ -1,29 +1,21 @@
 #/*##########################################################################
-# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2013 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
 #
-# This toolkit is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by the Free
+# This file is free software; you can redistribute it and/or modify it
+# under the terms of the GNU Lesser General Public License as published by the Free
 # Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
 #
-# PyMca is distributed in the hope that it will be useful, but WITHOUT ANY
+# This file is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 # details.
 #
-# You should have received a copy of the GNU General Public License along with
-# PyMca; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-#
-# PyMca follows the dual licensing model of Riverbank's PyQt and cannot be
-# used as a free plugin for a non-free program.
-#
-# Please contact the ESRF industrial unit (industry@esrf.fr) if this license
-# is a problem for you.
 #############################################################################*/
+__author__ = "V.A. Sole - ESRF Software Group"
 """
 Base class to handle stacks.
 
@@ -83,6 +75,11 @@ class StackBase(object):
         self.pluginList = []
         self.pluginInstanceDict = {}
         self.getPlugins()
+        # beyond 5 million elements, iterate to calculate the sums
+        # preventing huge intermediate use of memory when calculating
+        # the sums.
+        self._dynamicLimit = 5.0E6
+        self._tryNumpy = True
 
     def setPluginDirectoryList(self, dirlist):
         for directory in dirlist:
@@ -164,6 +161,12 @@ class StackBase(object):
         if id(stack) == id(self._stack):
             # just updated
             pass
+        elif hasattr(stack, "shape") and\
+             hasattr(stack, "dtype"):
+            # array like
+            self._stack.x = None
+            self._stack.data = stack
+            self._stack.info['SourceName'] = "Data of unknown origin"
         elif isinstance(stack, DataObject.DataObject) or\
            ("DataObject.DataObject" in ("%s" % type(stack))) or\
            ("QStack" in ("%s" % type(stack))) or\
@@ -211,22 +214,34 @@ class StackBase(object):
         """
         Recalculates the different images associated to the stack
         """
-        if isinstance(self._stack.data, numpy.ndarray):
-            self._stackImageData = numpy.sum(self._stack.data, self.mcaIndex)
+        self._tryNumpy = True
+        if hasattr(self._stack.data, "size"):
+            if self._stack.data.size >= self._dynamicLimit:
+                self._tryNumpy = False
+        else:
+            # is not a numpy ndarray in any case
+            self._tryNumpy = False
+
+        if self._tryNumpy and isinstance(self._stack.data, numpy.ndarray):
+            self._stackImageData = numpy.sum(self._stack.data,
+                                             axis=self.mcaIndex,
+                                             dtype=numpy.float)
             #original ICR mca
             if DEBUG:
                 print("(self.otherIndex, self.fileIndex) = (%d, %d)" %\
                       (self.otherIndex, self.fileIndex))
             i = max(self.otherIndex, self.fileIndex)
             j = min(self.otherIndex, self.fileIndex)
-            mcaData0 = numpy.sum(numpy.sum(self._stack.data, i), j) * 1.0
+            mcaData0 = numpy.sum(numpy.sum(self._stack.data,
+                                           axis=i,
+                                           dtype=numpy.float), j)
         else:
             if DEBUG:
                 t0 = time.time()
             shape = self._stack.data.shape
             if self.mcaIndex in [2, -1]:
                 self._stackImageData = numpy.zeros((shape[0], shape[1]),
-                                                self._stack.data.dtype)
+                                                dtype=numpy.float)
                 mcaData0 = numpy.zeros((shape[2],), numpy.float)
                 step = 1
                 if hasattr(self._stack, "monitor"):
@@ -243,7 +258,7 @@ class StackBase(object):
                     numpy.add(mcaData0, numpy.sum(tmpData, 0), mcaData0)
             elif self.mcaIndex == 0:
                 self._stackImageData = numpy.zeros((shape[1], shape[2]),
-                                                self._stack.data.dtype)
+                                                dtype=numpy.float)
                 mcaData0 = numpy.zeros((shape[0],), numpy.float)
                 step = 1
                 for i in range(shape[0]):
@@ -450,8 +465,12 @@ class StackBase(object):
             return
         mcaData = None
         goodData = numpy.isfinite(self._mcaData0.y[0].sum())
+        if DEBUG:
+            print("Stack data is not finite")
         if (self._selectionMask is None) and goodData:
             if normalize:
+                if DEBUG:
+                    print("Case 1")
                 npixels = self._stackImageData.shape[0] *\
                           self._stackImageData.shape[1] * 1.0
                 dataObject = DataObject.DataObject()
@@ -459,24 +478,38 @@ class StackBase(object):
                 dataObject.x = [self._mcaData0.x[0]]
                 dataObject.y = [self._mcaData0.y[0] / npixels];
             else:
+                if DEBUG:
+                    print("Case 2")
                 dataObject = self._mcaData0
             return dataObject
 
         #deal with NaN and inf values
         if self._selectionMask is None:
-            actualSelectionMask = numpy.isfinite(self._stackImageData)
+            if (self._ROIImageDict["ROI"] is not None) and\
+               (self.mcaIndex != 0):
+                actualSelectionMask = numpy.isfinite(self._ROIImageDict["ROI"])
+            else:
+                actualSelectionMask = numpy.isfinite(self._stackImageData)
         else:
-            actualSelectionMask = self._selectionMask * numpy.isfinite(self._stackImageData)
+            if (self._ROIImageDict["ROI"] is not None) and\
+               (self.mcaIndex != 0):
+                actualSelectionMask = self._selectionMask * numpy.isfinite(self._ROIImageDict["ROI"])
+            else:
+                actualSelectionMask = self._selectionMask * numpy.isfinite(self._stackImageData)
 
         npixels = actualSelectionMask.sum()
         if (npixels == 0) and goodData:
             if normalize:
+                if DEBUG:
+                    print("Case 3")
                 npixels = self._stackImageData.shape[0] * self._stackImageData.shape[1] * 1.0
                 dataObject = DataObject.DataObject()
                 dataObject.info.update(self._mcaData0.info)
                 dataObject.x = [self._mcaData0.x[0]]
                 dataObject.y = [self._mcaData0.y[0] / npixels]
             else:
+                if DEBUG:
+                    print("Case 4")
                 dataObject = self._mcaData0
             return dataObject
 
@@ -492,6 +525,8 @@ class StackBase(object):
         else:
                 arrayMask = (actualSelectionMask > 0)
 
+        if DEBUG:
+            print("Reached MCA calculation")
         cleanMask = numpy.nonzero(arrayMask)
         if DEBUG:
             print("self.fileIndex, self.mcaIndex = %d , %d" %\
@@ -499,10 +534,14 @@ class StackBase(object):
         if DEBUG:
             t0 = time.time()
         if len(cleanMask[0]) and len(cleanMask[1]):
+            if DEBUG:
+                print("USING MASK")
             cleanMask = numpy.array(cleanMask).transpose()
             if self.fileIndex == 2:
                 if self.mcaIndex == 0:
                     if isinstance(self._stack.data, numpy.ndarray):
+                        if DEBUG:
+                            print("In memory case 0")
                         for r, c in cleanMask:
                             mcaData += self._stack.data[:, r, c]
                     else:
@@ -533,6 +572,8 @@ class StackBase(object):
             elif self.fileIndex == 1:
                 if self.mcaIndex == 0:
                     if isinstance(self._stack.data, numpy.ndarray):
+                        if DEBUG:
+                            print("In memory case 2")
                         for r, c in cleanMask:
                             mcaData += self._stack.data[:, r, c]
                     else:
@@ -562,55 +603,65 @@ class StackBase(object):
                                 mcaData[i] = (tmpData[0] * arrayMask).sum(dtype=numpy.float)
                 elif self.mcaIndex == 2:
                     if isinstance(self._stack.data, numpy.ndarray):
+                        if DEBUG:
+                            print("In memory case 3")
                         for r, c in cleanMask:
                             mcaData += self._stack.data[r, c, :]
                     else:
                         if DEBUG:
                             print("Dynamic loading case 3")
                         #try to minimize access to the file
+                        row_list = []
                         row_dict = {}
                         for r, c in cleanMask:
-                            if r not in row_dict.keys():
-                                key = '%d' % r
-                                row_dict[key] = []
-                            row_dict[key].append(c)
-                        for key in row_dict.keys():
-                            r = int(key)
-                            tmpMcaData = self._stack.data[r:r + 1, row_dict[key], :]
-                            tmpMcaData.shape = 1, -1
+                            if r not in row_list:
+                                row_list.append(r)
+                                row_dict[r] = []
+                            row_dict[r].append(c)
+                        for r in row_list:
+                            tmpMcaData = self._stack.data[r:r + 1, row_dict[r], :]
+                            tmpMcaData.shape = -1, mcaData.shape[0]
                             mcaData += numpy.sum(tmpMcaData, axis=0, dtype=numpy.float)
                 else:
                     raise IndexError("Wrong combination of indices. Case 1")
             elif self.fileIndex == 0:
                 if self.mcaIndex == 1:
                     if isinstance(self._stack.data, numpy.ndarray):
+                        if DEBUG:
+                            print("In memory case 4")
                         for r, c in cleanMask:
                             mcaData += self._stack.data[r, :, c]
                     else:
                         raise IndexError("Dynamic loading case 4")
-                elif self.mcaIndex == 2:
+                elif self.mcaIndex in [2, -1]:
                     if isinstance(self._stack.data, numpy.ndarray):
+                        if DEBUG:
+                            print("In memory case 5")
                         for r, c in cleanMask:
                             mcaData += self._stack.data[r, c, :]
                     else:
                         if DEBUG:
                             print("Dynamic loading case 5")
                         #try to minimize access to the file
+                        row_list = []
                         row_dict = {}
                         for r, c in cleanMask:
-                            if r not in row_dict.keys():
-                                key = '%d' % r
-                                row_dict[key] = []
-                            row_dict[key].append(c)
-                        for key in row_dict.keys():
-                            r = int(key)
-                            tmpMcaData = self._stack.data[r:r + 1, row_dict[key], :]
-                            tmpMcaData.shape = 1, -1
-                            mcaData += numpy.sum(tmpMcaData, axis=0, dtype=numpy.float)
+                            if r not in row_list:
+                                row_list.append(r)
+                                row_dict[r] = []
+                            row_dict[r].append(c)
+                        for r in row_list:
+                            tmpMcaData = self._stack.data[r:r + 1, row_dict[r], :]
+                            tmpMcaData.shape = -1, mcaData.shape[0]
+                            mcaData += tmpMcaData.sum(axis=0, dtype=numpy.float)
                 else:
                     raise IndexError("Wrong combination of indices. Case 2")
             else:
                 raise IndexError("File index undefined")
+        else:
+            if DEBUG:
+                print("NOT USING MASK !")
+
         if DEBUG:
             print("Mca sum elapsed = %f" % (time.time() - t0))
         if goodData:
@@ -632,6 +683,8 @@ class StackBase(object):
         return dataObject
 
     def calculateROIImages(self, index1, index2, imiddle=None, energy=None):
+        if DEBUG:
+            print("Calculating ROI images")
         i1 = min(index1, index2)
         i2 = max(index1, index2)
         if imiddle is None:
@@ -665,7 +718,8 @@ class StackBase(object):
             else:
                 if DEBUG:
                     t0 = time.time()
-                if isinstance(self._stack.data, numpy.ndarray):
+                if self._tryNumpy and\
+                   isinstance(self._stack.data, numpy.ndarray):
                     leftImage = self._stack.data[:, :, i1]
                     middleImage = self._stack.data[:, :, imiddle]
                     rightImage = self._stack.data[:, :, i2 - 1]
@@ -675,9 +729,13 @@ class StackBase(object):
                     maxImage = energy[numpy.argmax(dataImage, axis=2) + i1]
                     minImage = energy[numpy.argmin(dataImage, axis=2) + i1]
                     isUsingSuppliedEnergyAxis = True
+                    if DEBUG:
+                        print("1 ROI image calculation elapsed = %f " %\
+                              (time.time() - t0))
                 else:
                     shape = self._stack.data.shape
-                    roiImage = self._stackImageData * 0
+                    roiImage = numpy.zeros(self._stackImageData.shape,
+                                           numpy.float)
                     background = roiImage * 1
                     leftImage = roiImage * 1
                     middleImage = roiImage * 1
@@ -700,32 +758,39 @@ class StackBase(object):
                         middleImage[i:i + step, :] += tmpData[:, :, imiddle - i1]
                         rightImage[i:i + step, :] += tmpData[:, :, -1]
                     background = 0.5 * (i2 - i1) * (leftImage + rightImage)
-                if DEBUG:
-                    print("ROI image calculation elapsed = %f " %\
-                          (time.time() - t0))
+                    if DEBUG:
+                        print("2 Dynamic ROI image calculation elapsed = %f " %\
+                              (time.time() - t0))
         elif self.fileIndex == 1:
             if self.mcaIndex == 0:
-                if isinstance(self._stack.data, numpy.ndarray):
+                if DEBUG:
+                    t0 = time.time()
+                if isinstance(self._stack.data, numpy.ndarray) and\
+                   self._tryNumpy:
                     leftImage = self._stack.data[i1, :, :]
                     middleImage= self._stack.data[imiddle, :, :]
                     rightImage = self._stack.data[i2 - 1, :, :]
                     dataImage = self._stack.data[i1:i2, :, :]
+                    # this calculation is very slow but it is extremely useful
+                    # for XANES studies
                     maxImage = energy[numpy.argmax(dataImage, axis=0) + i1]
                     minImage = energy[numpy.argmin(dataImage, axis=0) + i1]
                     isUsingSuppliedEnergyAxis = True
                     background = 0.5 * (i2 - i1) * (leftImage + rightImage)
                     roiImage = numpy.sum(dataImage, axis=0, dtype=numpy.float)
+                    if DEBUG:
+                        print("3 ROI image calculation elapsed = %f " %\
+                              (time.time() - t0))
                 else:
                     shape = self._stack.data.shape
-                    roiImage = self._stackImageData * 0
+                    roiImage = numpy.zeros(self._stackImageData.shape,
+                                           numpy.float)
                     background = roiImage * 1
                     leftImage = roiImage * 1
                     middleImage = roiImage * 1
                     rightImage = roiImage * 1
                     maxImage = roiImage * 1
                     minImage = roiImage * 1
-                    if DEBUG:
-                        t0 = time.time()
                     istep = 1
                     for i in range(i1, i2):
                         tmpData = self._stack.data[i:i + istep]
@@ -743,15 +808,16 @@ class StackBase(object):
                             middleImage = tmpData
                         elif i == (i2 - 1):
                             rightImage = tmpData
-                    if DEBUG:
-                        print("Dynamic ROI elapsed = %f" %\
-                              (time.time() - t0))
                     if i2 > i1:
                         background = (leftImage + rightImage) * 0.5 * (i2 - i1)
+                    if DEBUG:
+                        print("4 Dynamic ROI elapsed = %f" %\
+                              (time.time() - t0))
             else:
                 if DEBUG:
                     t0 = time.time()
-                if isinstance(self._stack.data, numpy.ndarray):
+                if self._tryNumpy and\
+                   isinstance(self._stack.data, numpy.ndarray):
                     leftImage = self._stack.data[:, :, i1]
                     middleImage = self._stack.data[:, :, imiddle]
                     rightImage = self._stack.data[:, :, i2 - 1]
@@ -761,9 +827,13 @@ class StackBase(object):
                     maxImage = energy[numpy.argmax(dataImage, axis=2) + i1]
                     minImage = energy[numpy.argmin(dataImage, axis=2) + i1]
                     isUsingSuppliedEnergyAxis = True
+                    if DEBUG:
+                        print("5 ROI Image elapsed = %f" %\
+                              (time.time() - t0))
                 else:
                     shape = self._stack.data.shape
-                    roiImage = self._stackImageData * 0
+                    roiImage = numpy.zeros(self._stackImageData.shape,
+                                           numpy.float)
                     background = roiImage * 1
                     leftImage = roiImage * 1
                     middleImage = roiImage * 1
@@ -786,11 +856,13 @@ class StackBase(object):
                         middleImage[i:i+step, :] += tmpData[:, :, imiddle-i1]
                         rightImage[i:i+step, :]  += tmpData[:, :,-1]
                     background = 0.5*(i2-i1)*(leftImage+rightImage)
-                if DEBUG:
-                    print("ROI image calculation elapsed = %f" %\
-                          (time.time() - t0))
+                    if DEBUG:
+                        print("6 Dynamic ROI image calculation elapsed = %f" %\
+                                          (time.time() - t0))
         else:
             #self.fileIndex = 2
+            if DEBUG:
+                t0 = time.time()
             if self.mcaIndex == 0:
                 leftImage = self._stack.data[i1]
                 middleImage = self._stack.data[imiddle]
@@ -801,6 +873,9 @@ class StackBase(object):
                 minImage = energy[numpy.argmin(dataImage, axis=0) + i1]
                 maxImage = energy[numpy.argmax(dataImage, axis=0) + i1]
                 isUsingSuppliedEnergyAxis = True
+                if DEBUG:
+                   print("7 Default ROI image calculation elapsed = %f" %\
+                                          (time.time() - t0))
             else:
                 leftImage = self._stack.data[:, i1, :]
                 middleImage = self._stack.data[:, imiddle, :]
@@ -811,7 +886,9 @@ class StackBase(object):
                 minImage = energy[numpy.argmin(dataImage, axis=1) + i1]
                 maxImage = energy[numpy.argmax(dataImage, axis=1) + i1]
                 isUsingSuppliedEnergyAxis = True
-
+                if DEBUG:
+                   print("8 Default ROI image calculation elapsed = %f" %\
+                                          (time.time() - t0))
 
         imageDict = {'ROI': roiImage,
                      'Maximum': maxImage,
@@ -821,6 +898,8 @@ class StackBase(object):
                      'Right': rightImage,
                      'Background': background}
         self.__ROIImageCalculationIsUsingSuppliedEnergyAxis = isUsingSuppliedEnergyAxis
+        if DEBUG:
+            print("ROI images calculated")
         return imageDict
 
     def setSelectionMask(self, mask):
@@ -831,7 +910,11 @@ class StackBase(object):
         if goodData:
             self._selectionMask = mask
         else:
-            self._selectionMask = mask * numpy.isfinite(self._stackImageData)
+            if (self._ROIImageDict["ROI"] is not None) and\
+               (self.mcaIndex != 0):
+                self._selectionMask = mask * numpy.isfinite(self._ROIImageDict["ROI"])
+            else:
+                self._selectionMask = mask * numpy.isfinite(self._stackImageData)
 
         for key in self.pluginInstanceDict.keys():
             self.pluginInstanceDict[key].selectionMaskUpdated()
