@@ -31,6 +31,7 @@ import copy
 from PyMca import PyMcaQt as qt
 from PyMca import PyMcaDirs
 from PyMca import DataObject
+from PyMca import MRCMap
 from PyMca import OmnicMap
 from PyMca import OpusDPTMap
 from PyMca import LuciaMap
@@ -55,7 +56,7 @@ class StackSelector(object):
     def __init__(self, parent=None):
         self.parent = parent
 
-    def getStack(self, filelist=None, imagestack=False):
+    def getStack(self, filelist=None, imagestack=None):
         if filelist in [None, []]:
             filelist, filefilter = self._getStackOfFiles(getfilter=True)
         else:
@@ -64,7 +65,7 @@ class StackSelector(object):
         if not len(filelist):
             return None
 
-        if filefilter == "":
+        if filefilter in ["", "All Files (*)"]:
             if HDF5:
                 if h5py.is_hdf5(filelist[0]):
                     filefilter = "HDF5"
@@ -82,7 +83,11 @@ class StackSelector(object):
                 if sys.version < '3.0':
                     line = f.read(10)
                 else:
-                    line = str(f.read(10).decode())
+                    try:
+                        line = str(f.read(10).decode())
+                    except UnicodeDecodeError:
+                        #give a dummy value
+                        line = "          "
                 f.close()
             omnicfile = False
             if filefilter.upper().startswith('HDF5'):
@@ -111,21 +116,34 @@ class StackSelector(object):
                   filelist[0].upper().endswith("TIFF")):
                 stack = TiffStack.TiffStack(imagestack=True)
             elif filefilter.upper().startswith("IMAGE"):
-                imagestack = True
+                if imagestack is None:
+                    imagestack = True
                 fileindex = 0
-                stack = QStack(imagestack=True)
+                stack = QStack(imagestack=imagestack)
             elif line[0] == "{":
                 if filelist[0].upper().endswith("RAW"):
-                    imagestack=True
+                    if imagestack is None:
+                        imagestack=True
                 stack = QStack(imagestack=imagestack)
             elif line[0:2] in ["II", "MM"]:
-                stack = QStack(imagestack=True)
+                if imagestack is None:
+                    imagestack = True
+                stack = QStack(imagestack=imagestack)
             elif line.startswith('Spectral'):
                 stack = OmnicMap.OmnicMap(filelist[0])
                 omnicfile = True
             elif line.startswith('#\tDate'):
                 stack = LuciaMap.LuciaMap(filelist[0])
                 omnicfile = True
+            elif filelist[0].upper().endswith("RAW.GZ")or\
+                 filelist[0].upper().endswith("EDF.GZ")or\
+                 filelist[0].upper().endswith("CCD.GZ")or\
+                 filelist[0].upper().endswith("RAW.BZ2")or\
+                 filelist[0].upper().endswith("EDF.BZ2")or\
+                 filelist[0].upper().endswith("CCD.BZ2"):
+                if imagestack is None:
+                    imagestack = True
+                stack = QStack(imagestack=imagestack)
             elif filelist[0][-4:].upper() in ["PIGE", "PIGE"]:
                 stack = SupaVisioMap.SupaVisioMap(filelist[0])
                 omnicfile = True
@@ -133,10 +151,14 @@ class StackSelector(object):
                 stack = SupaVisioMap.SupaVisioMap(filelist[0])
                 omnicfile = True
             elif filelist[0][-3:].upper() in ["SPE"] and\
-                 (line[0] not in ['$']):
+                 (line[0] not in ['$', '#']):
                 #Roper Scientific format
                 #handle it as MarCCD stack
                 stack = QStack(imagestack=True)
+            elif MRCMap.isMRCFile(filelist[0]):
+                stack = MRCMap.MRCMap(filelist[0])
+                omnicfile = True
+                imagestack = True
             else:
                 stack = QSpecFileStack()
 
@@ -231,6 +253,15 @@ class StackSelector(object):
                 raise IOError(\
                     "No HDF5 support while trying to read an HDF5 file")
             stack = QHDF5Stack1D.QHDF5Stack1D(args)
+        elif args[0].upper().endswith("RAW.GZ")or\
+             args[0].upper().endswith("EDF.GZ")or\
+             args[0].upper().endswith("CCD.GZ")or\
+             args[0].upper().endswith("RAW.BZ2")or\
+             args[0].upper().endswith("EDF.BZ2")or\
+             args[0].upper().endswith("CCD.BZ2"):
+            if imagestack is None:
+                imagestack = True
+            stack = QStack(imagestack=imagestack)            
         else:
             if HDF5:
                 if h5py.is_hdf5(args[0]):
@@ -352,17 +383,18 @@ class StackSelector(object):
         if getfilter is None:
             getfilter = False
         fileTypeList = ["EDF Files (*edf)",
+                        "Image Files (*edf *ccd *raw *edf.gz *ccd.gz *raw.gz)",
+                        "Image Files (*tif *tiff *TIF *TIFF)",
+                        "TextImage Files (*txt)",
+                        "HDF5 Files (*.nxs *.hdf *.h5)",
                         "EDF Files (*ccd)",
                         "Specfile Files (*mca)",
                         "Specfile Files (*dat)",
                         "OMNIC Files (*map)",
                         "OPUS-DPT Files (*.DPT *.dpt)",
-                        "HDF5 Files (*.nxs *.hdf *.h5)",
                         "AIFIRA Files (*DAT)",
                         "SupaVisio Files (*pige *pixe *rbs)",
-                        "Image Files (*edf *ccd *raw)",
-                        "Image Files (*tif *tiff *TIF *TIFF)",
-                        "TextImage Files (*txt)",
+                        "MRC files (*.mrc *.st)",
                         "All Files (*)"]
         if not HDF5:
             idx = fileTypeList.index("HDF5 Files (*.nxs *.hdf *.h5)")
