@@ -1,30 +1,20 @@
 #/*##########################################################################
-# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2013 European Synchrotron Radiation Facility
 #
-# This file is part of the PyMCA X-ray Fluorescence Toolkit developed at
-# the ESRF by the Beamline Instrumentation Software Support (BLISS) group.
+# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
+# the ESRF by the Software group.
 #
-# This toolkit is free software; you can redistribute it and/or modify it 
-# under the terms of the GNU General Public License as published by the Free
-# Software Foundation; either version 2 of the License, or (at your option) 
+# This file is free software; you can redistribute it and/or modify it
+# under the terms of the GNU Lesser General Public License as published by the Free
+# Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
 #
-# PyMCA is distributed in the hope that it will be useful, but WITHOUT ANY
+# This file is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 # details.
 #
-# You should have received a copy of the GNU General Public License along with
-# PyMCA; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
-# Suite 330, Boston, MA 02111-1307, USA.
-#
-# PyMCA follows the dual licensing model of Trolltech's Qt and Riverbank's PyQt
-# and cannot be used as a free plugin for a non-free program. 
-#
-# Please contact the ESRF industrial unit (industry@esrf.fr) if this license 
-# is a problem for you.
 #############################################################################*/
-/*char RcsId[] = "$Header: /segfs/bliss/source/python/specfile/specfile-3.1/src/RCS/sfdata.c,v 1.8 2005/07/04 15:02:38 ahoms Exp $"; */
 /************************************************************************
  *
  *   File:          sfdata.c
@@ -202,9 +192,9 @@ SfData( SpecFile *sf, long index, double ***retdata, long **retinfo, int *error
 
      char    strval[100];
      double  val;
-     double  valline[200];
+     double  valline[512];
      long    cols,
-             maxcol=200;
+             maxcol=512;
      long    rows;
      int     i;
 #ifndef _GNU_SOURCE
@@ -311,7 +301,6 @@ if(0){
 	setlocale(LC_NUMERIC, "C\0");
 #endif
 #endif
-
     for ( ; ptr < to; ptr++) {
         /* get a complete line */
         i=0;
@@ -319,8 +308,10 @@ if(0){
         /*I should be at the start of a line */ 
         while(*(ptr) != '\n'){
             if (*(ptr-1) == '\n'){
-                /*I am at the start of a line */ 
+                /*I am at the start of a line */
                 while(*ptr == '#'){
+                    if (ptr >= to)
+                        break;
                     for (ptr = ptr; ptr < to;ptr++){
                         if (*ptr == '\n'){
                             break;
@@ -329,7 +320,7 @@ if(0){
                     /* on exit is equal to newline */
                     if (ptr < to) {
                         ptr++;
-                    }              
+                    }
                 }
                 if (*ptr == '@') {
                      /*
@@ -344,6 +335,8 @@ if(0){
                     }
                 }
                 while(*ptr == '#'){
+                    if (ptr >= to)
+                        break;
                     for (ptr = ptr; ptr < to;ptr++){
                         if (*ptr == '\n'){
                             break;
@@ -352,7 +345,7 @@ if(0){
                     /* on exit is equal to newline */
                     if (ptr < to) {
                         ptr++;
-                    }              
+                    }
                 }
                 /* first characters of buffer
                 */
