@@ -1,6 +1,6 @@
 /****************************************************************************
 *
-*   Copyright (c) 1998-2011 European Synchrotron Radiation Facility (ESRF)
+*   Copyright (c) 1998-2013 European Synchrotron Radiation Facility (ESRF)
 *
 *   The software contained in this file "sps_py.c" is designed to interface
 *   the shared-data structures used and defined by the CSS "spec" package
@@ -30,6 +30,9 @@
 #include <sps.h>
 /* #include <stdio.h> */
 #include <Python.h>
+/* adding next line may raise errors ...
+#define NPY_NO_DEPRECATED_API NPY_1_7_API_VERSION
+*/
 #include <numpy/arrayobject.h>
 
 static PyObject *SPSError;
@@ -235,7 +238,7 @@ static PyObject *sps_attach(PyObject *self, PyObject *args)
 {
   char *spec_version, *array_name;
   int rows, cols, type, flag;
-  int dims[2];
+  npy_intp dims[2];
   int ptype, stype;
   PyArrayObject *arrobj;
   void *data;
@@ -304,7 +307,7 @@ static PyObject *sps_create(PyObject *self, PyObject *args)
 {
   char *spec_version, *array_name;
   int rows, cols, type = SPS_DOUBLE, flag = 0;
-  int dims[2];
+  npy_intp dims[2];
   int ptype, stype;
   PyArrayObject *arrobj;
   void *data;
@@ -448,7 +451,7 @@ static PyObject *sps_getdatacol(PyObject *self, PyObject *args)
 {
   char *spec_version, *array_name;
   int rows, cols, type, flag, in_row = 0, in_col;
-  int dims[2];
+  npy_intp dims[2];
   int ptype, stype;
   PyArrayObject *arrobj, *arrobj_nc;
 
@@ -519,7 +522,8 @@ static PyObject *sps_putdata(PyObject *self, PyObject *args)
     PyErr_SetString(SPSError, "Error copying data to shared memory");
     Py_DECREF(src);
     return NULL;
-  }
+  }else
+   Py_DECREF(src);
 
   Py_INCREF(Py_None);
   return Py_None;
@@ -563,7 +567,8 @@ static PyObject *sps_putdatarow(PyObject *self, PyObject *args)
     PyErr_SetString(SPSError, "Error copying data to shared memory");
     Py_DECREF(src);
     return NULL;
-  }
+  }else
+   Py_DECREF(src);
 
   Py_INCREF(Py_None);
   return Py_None;
@@ -601,7 +606,8 @@ static PyObject *sps_putdatacol(PyObject *self, PyObject *args)
     PyErr_SetString(SPSError, "Error copying data to shared memory");
     Py_DECREF(src);
     return NULL;
-  }
+  }else
+   Py_DECREF(src);
 
   Py_INCREF(Py_None);
   return Py_None;
