@@ -1,3 +1,23 @@
+#
+# These Python module have been developed by V.A. Sole, from the European
+# Synchrotron Radiation Facility (ESRF) to build a frozen version of PyMca.
+# Given the nature of this work, these module can be considered public domain. 
+# Therefore redistribution and use in source and binary forms, with or without
+# modification, are permitted provided the following disclaimer is accepted:
+#
+#
+# THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) AND THE ESRF ``AS IS'' AND ANY
+# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+# DISCLAIMED. IN NO EVENT SHALL THE AUTHOR(S) AND/OR THE ESRF BE LIABLE FOR ANY
+# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+#
+
 # A cx_freeze setup script to create PyMca executables
 #
 # Use "python cx_setup.py install"
@@ -152,6 +172,17 @@ try:
 except ImportError:
     MATPLOTLIB = False
 
+try:
+    import pyopencl
+    OPENCL = True
+    import sift
+except :
+    OPENCL = False
+
+if sys.platform.lower().startswith("linux"):
+    # no sense to freeze
+    OPENCL = False
+
 try:
     import mdp
     MDP = True
@@ -198,6 +229,12 @@ if OBJECT3D:
                   os.path.dirname(Object3D.__file__)]
     if H5PY_SPECIAL:
         special_modules.append(os.path.dirname(h5py.__file__))
+    if OPENCL:
+        special_modules.append(os.path.dirname(pyopencl.__file__))
+        excludes.append("PyMca.sift")
+        special_modules.append(os.path.dirname(sift.__file__))
+    else:
+        excludes.append("pyopencl")
     if MDP:
         #mdp versions above 2.5 need special treatment
         if mdp.__version__  > '2.5':
@@ -221,6 +258,12 @@ else:
     special_modules = [os.path.dirname(ctypes.__file__)]
     if H5PY_SPECIAL:
         special_modules.append(os.path.dirname(h5py.__file__))
+    if OPENCL:
+        excludes.append("PyMca.sift")
+        special_modules.append(os.path.dirname(sift.__file__))
+        special_modules.append(os.path.dirname(pyopencl.__file__))
+    else:
+        excludes.append("pyopencl")
     if MDP:
         #mdp versions above 2.5 need special treatment
         if mdp.__version__  > '2.5':
@@ -237,7 +280,6 @@ for f in ['qt', 'qttable', 'qtcanvas', 'Qwt5']:
 
 #Next line was for the plugins in frozen but now is in shared zip library
 #include_files.append((PyMcaDir, "PyMca"))
-
 buildOptions = dict(
         compressed = True,
         include_files = include_files,
@@ -297,7 +339,13 @@ for f in exec_list:
          
 executables = []
 for python_module in exec_list:
-    executables.append(Executable(os.path.join(PyMcaDir, python_module+".py")))
+    icon = None
+    # this allows to map a different icon to each executable
+    if sys.platform.startswith('win'):
+        if python_module in ["PyMcaMain", "QStackWidget"]:
+            icon = os.path.join(os.path.dirname(__file__), "icons", "PyMca.ico")
+    executables.append(Executable(os.path.join(PyMcaDir, python_module+".py"),
+                                  icon=icon))
 
 
 setup(
@@ -309,6 +357,36 @@ setup(
                        ),
         executables = executables)
 
+
+if OPENCL:
+    # pyopencl __init__.py needs to be patched
+    initFile = os.path.join(install_dir, "pyopencl", "__init__.py")
+    f = open(initFile, "r")
+    content = f.readlines()
+    f.close()
+    i = 0
+    i0 = 0
+    for line in content:
+        if "def _find_pyopencl_include_path():" in line:
+            i0 = i - 1
+        elif (i0 != 0) and ("}}}" in line):
+            i1 = i
+            break
+        i += 1
+    f = open(initFile, "w")
+    for i in range(0, i0):
+        f.write(content[i])
+    txt ='\n'
+    txt +='def _find_pyopencl_include_path():\n'
+    txt +='     from os.path import dirname, join, realpath\n'
+    txt +="     return '\"%s\"' % join(realpath(dirname(__file__)), \"cl\")"
+    txt +="\n"
+    txt +="\n"
+    f.write(txt)
+    for line in content[i1:]:
+        f.write(line)
+    f.close()
+
 if not sys.platform.startswith('win'):
     #rename the executables to .exe for easier handling by the start scripts
     for f in exec_list:
